@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -25,19 +25,23 @@ Copyright_License {
 #define XCSOAR_ANDROID_ENVIRONMENT_HPP
 
 #include <jni.h>
-#include <stddef.h>
+
+#include <cstddef>
+
+class AllocatedPath;
 
 namespace Environment {
-  void Initialise(JNIEnv *env);
-  void Deinitialise(JNIEnv *env);
 
-  /**
-   * Determine the mount point of the external SD card.
-   */
-  char *getExternalStorageDirectory(char *buffer, size_t max_size);
+void Initialise(JNIEnv *env);
+void Deinitialise(JNIEnv *env);
 
-  char *getExternalStoragePublicDirectory(char *buffer, size_t max_size,
-                                          const char *type);
-};
+/**
+ * Determine the mount point of the external SD card.
+ */
+AllocatedPath getExternalStorageDirectory() noexcept;
+
+AllocatedPath getExternalStoragePublicDirectory(const char *type) noexcept;
+
+} // namespace Environment
 
 #endif
