@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -33,22 +33,24 @@ Copyright_License {
 #include "Port/ConfiguredPort.hpp"
 #include "Port/DumpPort.hpp"
 #include "NMEA/Info.hpp"
-#include "Thread/Mutex.hpp"
-#include "Util/StringUtil.hpp"
-#include "Util/StringAPI.hxx"
+#include "thread/Mutex.hxx"
+#include "util/StringAPI.hxx"
+#include "util/ConvertString.hpp"
+#include "util/Exception.hxx"
 #include "Logger/NMEALogger.hpp"
 #include "Language/Language.hpp"
 #include "Operation/Operation.hpp"
-#include "OS/Clock.hpp"
+#include "system/Path.hpp"
 #include "../Simulator.hpp"
 #include "Input/InputQueue.hpp"
 #include "LogFile.hpp"
 #include "Job/Job.hpp"
 
 #ifdef ANDROID
-#include "Java/Object.hxx"
-#include "Java/Global.hxx"
+#include "java/Object.hxx"
+#include "java/Global.hxx"
 #include "Android/InternalSensors.hpp"
+#include "Android/GliderLink.hpp"
 #include "Android/Main.hpp"
 #include "Android/Product.hpp"
 #include "Android/IOIOHelper.hpp"
@@ -62,7 +64,7 @@ Copyright_License {
 #include "Apple/InternalSensors.hpp"
 #endif
 
-#include <assert.h>
+#include <cassert>
 
 /**
  * This scope class calls DeviceDescriptor::Return() and
@@ -96,9 +98,11 @@ public:
   };
 };
 
-DeviceDescriptor::DeviceDescriptor(unsigned _index,
+DeviceDescriptor::DeviceDescriptor(EventLoop &_event_loop,
+                                   Cares::Channel &_cares,
+                                   unsigned _index,
                                    PortListener *_port_listener)
-  :index(_index),
+  :event_loop(_event_loop), cares(_cares), index(_index),
    port_listener(_port_listener),
    open_job(nullptr),
    port(nullptr), monitor(nullptr), dispatcher(nullptr),
@@ -110,6 +114,7 @@ DeviceDescriptor::DeviceDescriptor(unsigned _index,
    droidsoar_v2(nullptr),
    nunchuck(nullptr),
    voltage(nullptr),
+   glider_link(nullptr),
 #endif
    n_failures(0u),
    ticker(false), borrowed(false)
@@ -122,6 +127,11 @@ DeviceDescriptor::DeviceDescriptor(unsigned _index,
 #endif
 }
 
+DeviceDescriptor::~DeviceDescriptor() noexcept
+{
+  assert(!IsOccupied());
+}
+
 void
 DeviceDescriptor::SetConfig(const DeviceConfig &_config)
 {
@@ -175,6 +185,9 @@ DeviceDescriptor::GetState() const
 
   if (voltage != nullptr)
     return PortState::READY;
+
+  if (glider_link != nullptr)
+    return PortState::READY;
 #endif
 
   return PortState::FAILED;
@@ -194,10 +207,10 @@ DeviceDescriptor::DisableDump()
 }
 
 void
-DeviceDescriptor::EnableDumpTemporarily(unsigned duration_ms)
+DeviceDescriptor::EnableDumpTemporarily(std::chrono::steady_clock::duration duration) noexcept
 {
   if (port != nullptr)
-    port->EnableTemporarily(duration_ms);
+    port->EnableTemporarily(duration);
 }
 
 bool
@@ -217,19 +230,20 @@ DeviceDescriptor::CancelAsync()
   assert(open_job != nullptr);
 
   async.Cancel();
-  async.Wait();
+
+  try {
+    async.Wait();
+  } catch (...) {
+    LogError(std::current_exception());
+  }
 
   delete open_job;
   open_job = nullptr;
 }
 
-bool
-DeviceDescriptor::OpenOnPort(DumpPort *_port, OperationEnvironment &env)
+inline bool
+DeviceDescriptor::OpenOnPort(std::unique_ptr<DumpPort> &&_port, OperationEnvironment &env)
 {
-#if !CLANG_CHECK_VERSION(3,6)
-  /* disabled on clang due to -Wtautological-pointer-compare */
-  assert(_port != nullptr);
-#endif
   assert(port == nullptr);
   assert(device == nullptr);
   assert(second_device == nullptr);
@@ -239,16 +253,17 @@ DeviceDescriptor::OpenOnPort(DumpPort *_port, OperationEnvironment &env)
 
   reopen_clock.Update();
 
-  device_blackboard->mutex.Lock();
-  device_blackboard->SetRealState(index).Reset();
-  device_blackboard->ScheduleMerge();
-  device_blackboard->mutex.Unlock();
+  {
+    const std::lock_guard<Mutex> lock(device_blackboard->mutex);
+    device_blackboard->SetRealState(index).Reset();
+    device_blackboard->ScheduleMerge();
+  }
 
   settings_sent.Clear();
   settings_received.Clear();
   was_alive = false;
 
-  port = _port;
+  port = std::move(_port);
 
   parser.Reset();
   parser.SetReal(!StringIsEqual(driver->name, _T("Condor")));
@@ -258,7 +273,7 @@ DeviceDescriptor::OpenOnPort(DumpPort *_port, OperationEnvironment &env)
   if (driver->CreateOnPort != nullptr) {
     Device *new_device = driver->CreateOnPort(config, *port);
 
-    const ScopeLock protect(mutex);
+    const std::lock_guard<Mutex> lock(mutex);
     device = new_device;
 
     if (driver->HasPassThrough() && config.use_second_device)
@@ -326,8 +341,9 @@ DeviceDescriptor::OpenDroidSoarV2()
     i2cbaro[1] = new I2CbaroDevice(GetIndex(), Java::GetEnv(),
                        ioio_helper->GetHolder(),
                        // needs calibration ?
-                       (config.sensor_factor == fixed(0)) ? DeviceConfig::PressureUse::PITOT_ZERO :
-                                                            DeviceConfig::PressureUse::PITOT,
+                       config.sensor_factor == 0
+                       ? DeviceConfig::PressureUse::PITOT_ZERO
+                       : DeviceConfig::PressureUse::PITOT,
                        config.sensor_offset, 1 + (0x77 << 8) + (46 << 16), 0 ,
                        5,
                        0);
@@ -352,9 +368,9 @@ DeviceDescriptor::OpenI2Cbaro()
       i2cbaro[i] = new I2CbaroDevice(GetIndex(), Java::GetEnv(),
                        ioio_helper->GetHolder(),
                        // needs calibration ?
-                       (config.sensor_factor == fixed(0) && config.press_use == DeviceConfig::PressureUse::PITOT) ?
-                                          DeviceConfig::PressureUse::PITOT_ZERO :
-                                          config.press_use,
+                       config.sensor_factor == 0 && config.press_use == DeviceConfig::PressureUse::PITOT
+                       ? DeviceConfig::PressureUse::PITOT_ZERO
+                       : config.press_use,
                        config.sensor_offset,
                        config.i2c_bus, config.i2c_addr,
                        config.press_use == DeviceConfig::PressureUse::TEK_PRESSURE ? 20 : 5,
@@ -406,10 +422,31 @@ DeviceDescriptor::OpenVoltage()
 }
 
 bool
-DeviceDescriptor::DoOpen(OperationEnvironment &env)
+DeviceDescriptor::OpenGliderLink()
 {
+#ifdef ANDROID
+  if (is_simulator())
+    return true;
+
+  glider_link = GliderLink::create(Java::GetEnv(), context, GetIndex());
+
+  return true;
+#else
+  return false;
+#endif
+}
+
+
+bool
+DeviceDescriptor::DoOpen(OperationEnvironment &env) noexcept
+try {
   assert(config.IsAvailable());
 
+  {
+    std::lock_guard<Mutex> lock(mutex);
+    error_message.clear();
+  }
+
   if (config.port_type == DeviceConfig::PortType::INTERNAL)
     return OpenInternalSensors();
 
@@ -425,9 +462,37 @@ DeviceDescriptor::DoOpen(OperationEnvironment &env)
   if (config.port_type == DeviceConfig::PortType::IOIOVOLTAGE)
     return OpenVoltage();
 
+  if (config.port_type == DeviceConfig::PortType::GLIDER_LINK)
+    return OpenGliderLink();
+
   reopen_clock.Update();
 
-  Port *port = OpenPort(config, port_listener, *this);
+  std::unique_ptr<Port> port;
+  try {
+    port = OpenPort(event_loop, cares, config, this, *this);
+  } catch (...) {
+    const auto e = std::current_exception();
+
+    TCHAR name_buffer[64];
+    const TCHAR *name = config.GetPortName(name_buffer, 64);
+
+    LogError(e, WideToUTF8Converter(name));
+
+    StaticString<256> msg;
+
+    const UTF8ToWideConverter what(GetFullMessage(e).c_str());
+    if (what.IsValid()) {
+      std::lock_guard<Mutex> lock(mutex);
+      error_message = what;
+    }
+
+    msg.Format(_T("%s: %s (%s)"), _("Unable to open port"), name,
+               (const TCHAR *)what);
+
+    env.SetErrorMessage(msg);
+    return false;
+  }
+
   if (port == nullptr) {
     TCHAR name_buffer[64];
     const TCHAR *name = config.GetPortName(name_buffer, 64);
@@ -438,19 +503,29 @@ DeviceDescriptor::DoOpen(OperationEnvironment &env)
     return false;
   }
 
-  DumpPort *dump_port = new DumpPort(port);
+  if (!port->WaitConnected(env)) {
+    if (!env.IsCancelled())
+      ++n_failures;
+
+    return false;
+  }
+
+  auto dump_port = std::make_unique<DumpPort>(std::move(port));
   dump_port->Disable();
 
-  if (!port->WaitConnected(env) || !OpenOnPort(dump_port, env)) {
+  if (!OpenOnPort(std::move(dump_port), env)) {
     if (!env.IsCancelled())
       ++n_failures;
 
-    delete dump_port;
     return false;
   }
 
   ResetFailureCounter();
   return true;
+} catch (...) {
+  const UTF8ToWideConverter msg(GetFullMessage(std::current_exception()).c_str());
+  env.SetErrorMessage(msg);
+  return false;
 }
 
 void
@@ -475,7 +550,7 @@ DeviceDescriptor::Open(OperationEnvironment &env)
   LogFormat(_T("Opening device %s"), config.GetPortName(buffer, 64));
 
   open_job = new OpenDeviceJob(*this);
-  async.Start(open_job, env, this);
+  async.Start(open_job, env, &job_finished_notify);
 }
 
 void
@@ -505,13 +580,15 @@ DeviceDescriptor::Close()
   delete voltage;
   voltage = nullptr;
 
+  delete glider_link;
+  glider_link = nullptr;
 #endif
 
   /* safely delete the Device object */
   Device *old_device = device;
 
   {
-    const ScopeLock protect(mutex);
+    const std::lock_guard<Mutex> lock(mutex);
     device = nullptr;
     /* after leaving this scope, no other thread may use the old
        object; to avoid locking the mutex for too long, the "delete"
@@ -523,16 +600,15 @@ DeviceDescriptor::Close()
   delete second_device;
   second_device = nullptr;
 
-  Port *old_port = port;
-  port = nullptr;
-  delete old_port;
+  port.reset();
 
   ticker = false;
 
-  device_blackboard->mutex.Lock();
-  device_blackboard->SetRealState(index).Reset();
-  device_blackboard->ScheduleMerge();
-  device_blackboard->mutex.Unlock();
+  {
+    const std::lock_guard<Mutex> lock(device_blackboard->mutex);
+    device_blackboard->SetRealState(index).Reset();
+    device_blackboard->ScheduleMerge();
+  }
 
   settings_sent.Clear();
   settings_received.Clear();
@@ -558,29 +634,9 @@ DeviceDescriptor::AutoReopen(OperationEnvironment &env)
       !config.IsAvailable() ||
       !ShouldReopen() ||
       /* attempt to reopen a failed device every 30 seconds */
-      !reopen_clock.CheckUpdate(30000))
+      !reopen_clock.CheckUpdate(std::chrono::seconds(30)))
     return;
 
-#ifdef ANDROID
-  if (config.port_type == DeviceConfig::PortType::RFCOMM &&
-      android_api_level < 11 && n_failures >= 2) {
-    /* on Android < 3.0, system_server's "BT EventLoop" thread
-       eventually crashes with JNI reference table overflow due to a
-       memory leak after too many Bluetooth failures
-       (https://code.google.com/p/android/issues/detail?id=8676);
-       don't attempt to reconnect on this Android version over and
-       over to keep the chance of this bug occurring low enough */
-
-    if (n_failures == 2) {
-      LogFormat(_T("Giving up on Bluetooth device %s to avoid Android crash bug"),
-                config.bluetooth_mac.c_str());
-      ++n_failures;
-    }
-
-    return;
-  }
-#endif
-
   TCHAR buffer[64];
   LogFormat(_T("Reconnecting to device %s"), config.GetPortName(buffer, 64));
 
@@ -687,7 +743,7 @@ DeviceDescriptor::Return()
 bool
 DeviceDescriptor::IsAlive() const
 {
-  ScopeLock protect(device_blackboard->mutex);
+  std::lock_guard<Mutex> lock(device_blackboard->mutex);
   return device_blackboard->RealState(index).alive;
 }
 
@@ -735,7 +791,7 @@ DeviceDescriptor::ForwardLine(const char *line)
      any thread, and if the Port gets closed, bad things happen */
 
   if (IsNMEAOut() && port != nullptr) {
-    Port *p = port;
+    Port *p = port.get();
     p->Write(line);
     p->Write("\r\n");
   }
@@ -758,7 +814,7 @@ DeviceDescriptor::WriteNMEA(const TCHAR *line, OperationEnvironment &env)
   if (port == nullptr)
     return false;
 
-  char buffer[_tcslen(line) * 4 + 1];
+  char buffer[4096];
   if (::WideCharToMultiByte(CP_ACP, 0, line, -1, buffer, sizeof(buffer),
                             nullptr, nullptr) <= 0)
     return false;
@@ -768,7 +824,7 @@ DeviceDescriptor::WriteNMEA(const TCHAR *line, OperationEnvironment &env)
 #endif
 
 bool
-DeviceDescriptor::PutMacCready(fixed value, OperationEnvironment &env)
+DeviceDescriptor::PutMacCready(double value, OperationEnvironment &env)
 {
   assert(InMainThread());
 
@@ -784,7 +840,7 @@ DeviceDescriptor::PutMacCready(fixed value, OperationEnvironment &env)
   if (!device->PutMacCready(value, env))
     return false;
 
-  ScopeLock protect(device_blackboard->mutex);
+  std::lock_guard<Mutex> lock(device_blackboard->mutex);
   NMEAInfo &basic = device_blackboard->SetRealState(index);
   settings_sent.mac_cready = value;
   settings_sent.mac_cready_available.Update(basic.clock);
@@ -793,7 +849,7 @@ DeviceDescriptor::PutMacCready(fixed value, OperationEnvironment &env)
 }
 
 bool
-DeviceDescriptor::PutBugs(fixed value, OperationEnvironment &env)
+DeviceDescriptor::PutBugs(double value, OperationEnvironment &env)
 {
   assert(InMainThread());
 
@@ -809,7 +865,7 @@ DeviceDescriptor::PutBugs(fixed value, OperationEnvironment &env)
   if (!device->PutBugs(value, env))
     return false;
 
-  ScopeLock protect(device_blackboard->mutex);
+  std::lock_guard<Mutex> lock(device_blackboard->mutex);
   NMEAInfo &basic = device_blackboard->SetRealState(index);
   settings_sent.bugs = value;
   settings_sent.bugs_available.Update(basic.clock);
@@ -818,7 +874,7 @@ DeviceDescriptor::PutBugs(fixed value, OperationEnvironment &env)
 }
 
 bool
-DeviceDescriptor::PutBallast(fixed fraction, fixed overload,
+DeviceDescriptor::PutBallast(double fraction, double overload,
                              OperationEnvironment &env)
 {
   assert(InMainThread());
@@ -836,7 +892,7 @@ DeviceDescriptor::PutBallast(fixed fraction, fixed overload,
   if (!device->PutBallast(fraction, overload, env))
     return false;
 
-  ScopeLock protect(device_blackboard->mutex);
+  std::lock_guard<Mutex> lock(device_blackboard->mutex);
   NMEAInfo &basic = device_blackboard->SetRealState(index);
   settings_sent.ballast_fraction = fraction;
   settings_sent.ballast_fraction_available.Update(basic.clock);
@@ -916,7 +972,7 @@ DeviceDescriptor::PutQNH(const AtmosphericPressure &value,
   if (!device->PutQNH(value, env))
     return false;
 
-  ScopeLock protect(device_blackboard->mutex);
+  std::lock_guard<Mutex> lock(device_blackboard->mutex);
   NMEAInfo &basic = device_blackboard->SetRealState(index);
   settings_sent.qnh = value;
   settings_sent.qnh_available.Update(basic.clock);
@@ -1021,7 +1077,7 @@ DeviceDescriptor::ReadFlightList(RecordedFlightList &flight_list,
 
 bool
 DeviceDescriptor::DownloadFlight(const RecordedFlightInfo &flight,
-                                 const TCHAR *path,
+                                 Path path,
                                  OperationEnvironment &env)
 {
   assert(borrowed);
@@ -1087,7 +1143,7 @@ DeviceDescriptor::OnSensorUpdate(const MoreData &basic)
   /* must hold the mutex because this method may run in any thread,
      just in case the main thread deletes the Device while this method
      still runs */
-  const ScopeLock protect(mutex);
+  const std::lock_guard<Mutex> lock(mutex);
 
   if (device != nullptr)
     device->OnSensorUpdate(basic);
@@ -1106,56 +1162,90 @@ DeviceDescriptor::OnCalculatedUpdate(const MoreData &basic,
 bool
 DeviceDescriptor::ParseLine(const char *line)
 {
-  ScopeLock protect(device_blackboard->mutex);
+  std::lock_guard<Mutex> lock(device_blackboard->mutex);
   NMEAInfo &basic = device_blackboard->SetRealState(index);
   basic.UpdateClock();
   return ParseNMEA(line, basic);
 }
 
 void
-DeviceDescriptor::OnNotification()
+DeviceDescriptor::OnJobFinished() noexcept
 {
   /* notification from AsyncJobRunner, the Job was finished */
 
   assert(InMainThread());
   assert(open_job != nullptr);
 
-  async.Wait();
+  try {
+    async.Wait();
+  } catch (...) {
+    LogError(std::current_exception());
+  }
 
   delete open_job;
   open_job = nullptr;
 }
 
 void
-DeviceDescriptor::DataReceived(const void *data, size_t length)
+DeviceDescriptor::PortStateChanged() noexcept
+{
+  if (port_listener != nullptr)
+    port_listener->PortStateChanged();
+}
+
+void
+DeviceDescriptor::PortError(const char *msg) noexcept
+{
+  {
+    TCHAR buffer[64];
+    LogFormat(_T("Error on device %s: %s"),
+              config.GetPortName(buffer, 64), msg);
+  }
+
+  {
+    const UTF8ToWideConverter tmsg(msg);
+    if (tmsg.IsValid()) {
+      std::lock_guard<Mutex> lock(mutex);
+      error_message = tmsg;
+    }
+  }
+
+  if (port_listener != nullptr)
+    port_listener->PortError(msg);
+}
+
+bool
+DeviceDescriptor::DataReceived(const void *data, size_t length) noexcept
 {
   if (monitor != nullptr)
     monitor->DataReceived(data, length);
 
   // Pass data directly to drivers that use binary data protocols
   if (driver != nullptr && device != nullptr && driver->UsesRawData()) {
-    ScopeLock protect(device_blackboard->mutex);
-    NMEAInfo &basic = device_blackboard->SetRealState(index);
-    basic.UpdateClock();
+    auto basic = device_blackboard->LockGetDeviceDataUpdateClock(index);
 
     const ExternalSettings old_settings = basic.settings;
 
+    /* call Device::DataReceived() without holding
+       DeviceBlackboard::mutex to avoid blocking all other threads */
     if (device->DataReceived(data, length, basic)) {
       if (!config.sync_from_device)
         basic.settings = old_settings;
 
-      device_blackboard->ScheduleMerge();
+      device_blackboard->LockSetDeviceDataScheuduleMerge(index, basic);
     }
 
-    return;
+    return true;
   }
 
   if (!IsNMEAOut())
     PortLineSplitter::DataReceived(data, length);
+
+  return true;
 }
 
-void
-DeviceDescriptor::LineReceived(const char *line)
+bool
+DeviceDescriptor::LineReceived(const char *line) noexcept
 {
   NMEALogger::Log(line);
 
@@ -1164,4 +1254,6 @@ DeviceDescriptor::LineReceived(const char *line)
 
   if (ParseLine(line))
     device_blackboard->ScheduleMerge();
+
+  return true;
 }
