@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,8 +24,9 @@ Copyright_License {
 #ifndef XCSOAR_DEVICE_DRIVER_LX_CONVERT_HPP
 #define XCSOAR_DEVICE_DRIVER_LX_CONVERT_HPP
 
-#include <stdio.h>
-#include <stddef.h>
+#include <cstddef>
+
+class BufferedOutputStream;
 
 namespace LX {
   /**
@@ -33,7 +34,7 @@ namespace LX {
    */
   bool
   ConvertLXNToIGC(const void *data, size_t length,
-                  FILE *file);
+                  BufferedOutputStream &os);
 }
 
 #endif
