@@ -3,7 +3,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,14 +24,13 @@ Copyright_License {
 */
 
 #include "Device/Parser.hpp"
-#include "Util/CharUtil.hpp"
 #include "Geo/Geoid.hpp"
 #include "NMEA/Info.hpp"
 #include "NMEA/Checksum.hpp"
 #include "NMEA/InputLine.hpp"
-#include "Util/StringUtil.hpp"
 #include "Units/System.hpp"
 #include "Driver/FLARM/StaticParser.hpp"
+#include "util/CharUtil.hxx"
 
 NMEAParser::NMEAParser()
 {
@@ -43,13 +42,13 @@ NMEAParser::Reset()
 {
   real = true;
   use_geoid = true;
-  last_time = fixed(0);
+  last_time = 0;
 }
 
 bool
 NMEAParser::ParseLine(const char *string, NMEAInfo &info)
 {
-  assert(positive(info.clock));
+  assert(info.clock > 0);
 
   if (string[0] != '$')
     return false;
@@ -77,6 +76,9 @@ NMEAParser::ParseLine(const char *string, NMEAInfo &info)
 
     if (StringIsEqual(type + 3, "HDM"))
       return HDM(line, info);
+
+    if (StringIsEqual(type + 3, "MWV"))
+      return MWV(line, info);
   }
 
   // if (proprietary sentence) ...
@@ -133,11 +135,11 @@ NAVWarn(char c)
 static bool
 ReadBearing(NMEAInputLine &line, Angle &value_r)
 {
-  fixed value;
+  double value;
   if (!line.ReadChecked(value))
     return false;
 
-  if (negative(value) || value > fixed(360))
+  if (value < 0 || value > 360)
     return false;
 
   value_r = Angle::Degrees(value).AsBearing();
@@ -167,12 +169,12 @@ ReadGeoAngle(NMEAInputLine &line, Angle &a)
   if (y < 0 || y > 180 || endptr == buffer || *endptr != 0)
     return false;
 
-  a = Angle::Degrees(fixed(y) + fixed(x) / 60);
+  a = Angle::Degrees(y + x / 60.);
   return true;
 }
 
 static bool
-ReadFixedAndChar(NMEAInputLine &line, fixed &d, char &ch)
+ReadDoubleAndChar(NMEAInputLine &line, double &d, char &ch)
 {
   bool success = line.ReadChecked(d);
   ch = line.ReadFirstChar();
@@ -231,11 +233,11 @@ NMEAParser::ReadGeoPoint(NMEAInputLine &line, GeoPoint &value_r)
  * Reads an altitude value, and the unit from a second volumn.
  */
 static bool
-ReadAltitude(NMEAInputLine &line, fixed &value_r)
+ReadAltitude(NMEAInputLine &line, double &value_r)
 {
-  fixed value;
+  double value;
   char format;
-  if (!ReadFixedAndChar(line, value, format))
+  if (!ReadDoubleAndChar(line, value, format))
     return false;
 
   if (format == 'f' || format == 'F')
@@ -246,31 +248,31 @@ ReadAltitude(NMEAInputLine &line, fixed &value_r)
 }
 
 bool
-NMEAParser::TimeHasAdvanced(fixed this_time, NMEAInfo &info)
+NMEAParser::TimeHasAdvanced(double this_time, NMEAInfo &info)
 {
   return TimeHasAdvanced(this_time, last_time, info);
 }
 
 gcc_const
 static bool
-IsMidnightWraparound(fixed this_time, fixed last_time)
+IsMidnightWraparound(double this_time, double last_time)
 {
   constexpr unsigned SECONDS_PER_HOUR = 60 * 60;
   constexpr unsigned SECONDS_PER_DAY = 24 * SECONDS_PER_HOUR;
 
-  return this_time < fixed(SECONDS_PER_HOUR) &&
-                     last_time >= fixed(SECONDS_PER_DAY - SECONDS_PER_HOUR);
+  return this_time < SECONDS_PER_HOUR &&
+    last_time >= SECONDS_PER_DAY - SECONDS_PER_HOUR;
 }
 
 gcc_const
 static bool
-TimeHasAdvanced(fixed this_time, fixed last_time)
+TimeHasAdvanced(double this_time, double last_time)
 {
   return this_time >= last_time || IsMidnightWraparound(this_time, last_time);
 }
 
 bool
-NMEAParser::TimeHasAdvanced(fixed this_time, fixed &last_time, NMEAInfo &info)
+NMEAParser::TimeHasAdvanced(double this_time, double &last_time, NMEAInfo &info)
 {
   if (!::TimeHasAdvanced(this_time, last_time)) {
     last_time = this_time;
@@ -315,6 +317,10 @@ NMEAParser::GSA(NMEAInputLine &line, NMEAInfo &info)
 
   info.gps.satellite_ids_available.Update(info.clock);
 
+  info.gps.pdop = line.Read(-1.);
+  info.gps.hdop = line.Read(-1.);
+  info.gps.vdop = line.Read(-1.);
+
   return true;
 }
 
@@ -338,7 +344,7 @@ NMEAParser::GLL(NMEAInputLine &line, NMEAInfo &info)
   GeoPoint location;
   bool valid_location = ReadGeoPoint(line, location);
 
-  fixed this_time;
+  double this_time;
   if (!ReadTime(line, info.date_time_utc, this_time))
     return true;
 
@@ -389,7 +395,7 @@ NMEAParser::ReadDate(NMEAInputLine &line, BrokenDate &date)
 
 bool
 NMEAParser::ReadTime(NMEAInputLine &line, BrokenTime &broken_time,
-                     fixed &time_of_day_s)
+                     double &time_of_day_s)
 {
   double value;
   if (!line.ReadChecked(value) || value < 0)
@@ -411,7 +417,7 @@ NMEAParser::ReadTime(NMEAInputLine &line, BrokenTime &broken_time,
     return false;
 
   broken_time = BrokenTime(hour, minute, (unsigned)second);
-  time_of_day_s = fixed(hour * 3600 + minute * 60) + fixed(second);
+  time_of_day_s = (hour * 3600 + minute * 60) + second;
   return true;
 }
 
@@ -422,7 +428,7 @@ NMEAParser::ReadTime(NMEAInputLine &line, BrokenTime &broken_time,
 static bool
 ReadVariation(NMEAInputLine &line, Angle &value_r)
 {
-  fixed value;
+  double value;
   if (!line.ReadChecked(value))
     return false;
   char ch = line.ReadOneChar();
@@ -457,7 +463,7 @@ NMEAParser::RMC(NMEAInputLine &line, NMEAInfo &info)
    * 13) Checksum
    */
 
-  fixed this_time;
+  double this_time;
   if (!ReadTime(line, info.date_time_utc, this_time))
     return true;
 
@@ -466,7 +472,7 @@ NMEAParser::RMC(NMEAInputLine &line, NMEAInfo &info)
   GeoPoint location;
   bool valid_location = ReadGeoPoint(line, location);
 
-  fixed speed;
+  double speed;
   bool ground_speed_available = line.ReadChecked(speed);
 
   Angle track;
@@ -581,7 +587,7 @@ NMEAParser::GGA(NMEAInputLine &line, NMEAInfo &info)
 
   GPSState &gps = info.gps;
 
-  fixed this_time;
+  double this_time;
   if (!ReadTime(line, info.date_time_utc, this_time))
     return true;
 
@@ -619,7 +625,7 @@ NMEAParser::GGA(NMEAInputLine &line, NMEAInfo &info)
   info.gps.nonexpiring_internal_gps = false;
 #endif
 
-  gps.hdop = line.Read(fixed(0));
+  gps.hdop = line.Read(-1.);
 
   bool altitude_available = ReadAltitude(line, info.gps_altitude);
   if (altitude_available)
@@ -627,7 +633,7 @@ NMEAParser::GGA(NMEAInputLine &line, NMEAInfo &info)
   else
     info.gps_altitude_available.Clear();
 
-  fixed geoid_separation;
+  double geoid_separation;
   if (ReadAltitude(line, geoid_separation)) {
     // No real need to parse this value,
     // but we do assume that no correction is required in this case
@@ -663,7 +669,7 @@ NMEAParser::RMZ(NMEAInputLine &line, NMEAInfo &info)
 {
   //JMW?  RMZAltitude = info.pressure.PressureAltitudeToQNHAltitude(RMZAltitude);
 
-  fixed value;
+  double value;
   if (ReadAltitude(line, value)) {
     // JMW no in-built baro sources, so use this generic one
     if (info.flarm.IsDetected()) {
@@ -713,10 +719,10 @@ NMEAParser::PTAS1(NMEAInputLine &line, NMEAInfo &info)
    */
 
   // Parse current vario data
-  fixed vario;
+  double vario;
   if (line.ReadChecked(vario)) {
     // Properly convert to m/s
-    vario = Units::ToSysUnit((vario - fixed(200)) / 10, Unit::KNOTS);
+    vario = Units::ToSysUnit((vario - 200) / 10., Unit::KNOTS);
     info.ProvideTotalEnergyVario(vario);
   }
 
@@ -724,17 +730,66 @@ NMEAParser::PTAS1(NMEAInputLine &line, NMEAInfo &info)
   line.Skip();
 
   // Parse barometric altitude
-  fixed baro_altitude;
+  double baro_altitude;
   if (line.ReadChecked(baro_altitude)) {
     // Properly convert to meter
-    baro_altitude = Units::ToSysUnit(baro_altitude - fixed(2000), Unit::FEET);
+    baro_altitude = Units::ToSysUnit(baro_altitude - 2000, Unit::FEET);
     info.ProvidePressureAltitude(baro_altitude);
   }
 
   // Parse true airspeed
-  fixed vtas;
+  double vtas;
   if (line.ReadChecked(vtas))
     info.ProvideTrueAirspeed(Units::ToSysUnit(vtas, Unit::KNOTS));
 
   return true;
 }
+
+inline bool
+NMEAParser::MWV(NMEAInputLine &line, NMEAInfo &info)
+{
+  /*
+    * $--MWV,x.x,a,x.x,a,a,a,*hh
+    *
+    * Field Number:
+    *  1) wind angle
+    *  2) (R)elative or (T)rue
+    *  3) wind speed
+    *  4) K/M/N
+    *  5) Status A=valid
+    *  8) Checksum
+    */
+
+  Angle winddir;
+  if (!ReadBearing(line, winddir))
+    return false;
+
+  char ch = line.ReadOneChar();
+
+  double windspeed;
+  if (!line.ReadChecked(windspeed))
+    return false;
+
+  ch = line.ReadOneChar();
+  switch (ch) {
+  case 'N':
+    windspeed = Units::ToSysUnit(windspeed, Unit::KNOTS);
+    break;
+
+  case 'K':
+    windspeed = Units::ToSysUnit(windspeed, Unit::KILOMETER_PER_HOUR);
+    break;
+
+  case 'M':
+    windspeed = Units::ToSysUnit(windspeed, Unit::METER_PER_SECOND);
+    break;
+
+  default:
+    return false;
+  }
+
+  SpeedVector wind(winddir, windspeed);
+  info.ProvideExternalWind(wind);
+
+  return true;
+}
