@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -34,22 +34,21 @@ Copyright_License {
 #include "Dialogs/WidgetDialog.hpp"
 #include "Dialogs/Message.hpp"
 #include "UIGlobals.hpp"
-#include "Util/TrivialArray.hpp"
-#include "Util/StaticString.hxx"
-#include "Util/Macros.hpp"
+#include "util/StaticString.hxx"
+#include "util/Macros.hpp"
 #include "Device/MultipleDevices.hpp"
 #include "Device/Descriptor.hpp"
 #include "Device/Register.hpp"
 #include "Device/Port/Listener.hpp"
 #include "Device/Driver/LX/Internal.hpp"
-#include "Event/Notify.hpp"
+#include "ui/event/Notify.hpp"
 #include "Blackboard/DeviceBlackboard.hpp"
 #include "Blackboard/BlackboardListener.hpp"
 #include "Components.hpp"
 #include "Look/DialogLook.hpp"
 #include "Form/List.hpp"
 #include "Widget/ListWidget.hpp"
-#include "Screen/Canvas.hpp"
+#include "ui/canvas/Canvas.hpp"
 #include "Screen/Layout.hpp"
 #include "Language/Language.hpp"
 #include "Operation/MessageOperationEnvironment.hpp"
@@ -61,27 +60,25 @@ Copyright_License {
 #include "Interface.hpp"
 
 #ifdef ANDROID
-#include "Java/Global.hxx"
+#include "java/Global.hxx"
 #include "Android/BluetoothHelper.hpp"
 #endif
 
-class DeviceListWidget final
-  : public ListWidget, private ActionListener,
-    NullBlackboardListener, PortListener, Notify {
-  enum Buttons {
-    DISABLE,
-    RECONNECT, FLIGHT, EDIT, MANAGE, MONITOR,
-    DEBUG,
-  };
+using namespace UI;
 
+class DeviceListWidget final
+  : public ListWidget,
+    NullBlackboardListener, PortListener {
   const DialogLook &look;
 
-  UPixelScalar font_height;
+  unsigned font_height;
 
   struct Flags {
     bool duplicate:1;
     bool open:1, error:1;
     bool alive:1, location:1, gps:1, baro:1, airspeed:1, vario:1, traffic:1;
+    bool temperature:1;
+    bool humidity:1;
     bool debug:1;
 
     void Set(const DeviceConfig &config, const DeviceDescriptor &device,
@@ -117,6 +114,8 @@ class DeviceListWidget final
       airspeed = basic.airspeed_available;
       vario = basic.total_energy_vario_available;
       traffic = basic.flarm.IsDetected();
+      temperature = basic.temperature_available;
+      humidity = basic.humidity_available;
       debug = device.IsDumpEnabled();
     }
   };
@@ -159,6 +158,7 @@ class DeviceListWidget final
   static_assert(sizeof(Item) == 2, "wrong size");
 
   Item items[NUMDEV];
+  tstring error_messages[NUMDEV];
 
   Button *disable_button;
   Button *reconnect_button, *flight_button;
@@ -166,6 +166,11 @@ class DeviceListWidget final
   Button *manage_button, *monitor_button;
   Button *debug_button;
 
+  Notify port_state_notify{[this]{
+    if (RefreshList())
+      UpdateButtons();
+  }};
+
 public:
   DeviceListWidget(const DialogLook &_look)
     :look(_look) {}
@@ -187,12 +192,9 @@ protected:
 
 public:
   /* virtual methods from class Widget */
-  virtual void Prepare(ContainerWindow &parent, const PixelRect &rc) override;
-  virtual void Unprepare() override {
-    DeleteWindow();
-  }
+  void Prepare(ContainerWindow &parent, const PixelRect &rc) noexcept override;
 
-  virtual void Show(const PixelRect &rc) override {
+  void Show(const PixelRect &rc) noexcept override {
     ListWidget::Show(rc);
 
     devices->AddPortListener(*this);
@@ -202,7 +204,7 @@ public:
     UpdateButtons();
   }
 
-  virtual void Hide() override {
+  void Hide() noexcept override {
     ListWidget::Hide();
 
     CommonInterface::GetLiveBlackboard().RemoveListener(*this);
@@ -210,34 +212,26 @@ public:
   }
 
   /* virtual methods from class List::Handler */
-  virtual void OnPaintItem(Canvas &canvas, const PixelRect rc,
-                           unsigned idx) override;
-  virtual void OnCursorMoved(unsigned index) override;
+   void OnPaintItem(Canvas &canvas, const PixelRect rc,
+                    unsigned idx) noexcept override;
+  void OnCursorMoved(unsigned index) noexcept override;
 
 private:
-  /* virtual methods from class ActionListener */
-  virtual void OnAction(int id) override;
-
   /* virtual methods from class BlackboardListener */
   virtual void OnGPSUpdate(const MoreData &basic) override;
 
   /* virtual methods from class PortListener */
-  void PortStateChanged() override {
-    Notify::SendNotification();
-  }
-
-  /* virtual methods from class Notify */
-  void OnNotification() override {
-    if (RefreshList())
-      UpdateButtons();
+  void PortStateChanged() noexcept override {
+    port_state_notify.SendNotification();
   }
 };
 
 void
-DeviceListWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
+DeviceListWidget::Prepare(ContainerWindow &parent,
+                          const PixelRect &rc) noexcept
 {
   const DialogLook &look = UIGlobals::GetDialogLook();
-  const UPixelScalar margin = Layout::GetTextPadding();
+  const unsigned margin = Layout::GetTextPadding();
   font_height = look.list.font->GetHeight();
   CreateList(parent, look, rc, 3 * margin + font_height +
              look.small_font.GetHeight()).SetLength(NUMDEV);
@@ -263,6 +257,12 @@ DeviceListWidget::RefreshList()
       item = n;
       modified = true;
     }
+
+    auto error_message = (*devices)[i].GetErrorMessage();
+    if (error_message != error_messages[i]) {
+      error_messages[i] = std::move(error_message);
+      modified = true;
+    }
   }
 
   if (modified)
@@ -273,13 +273,33 @@ DeviceListWidget::RefreshList()
 void
 DeviceListWidget::CreateButtons(WidgetDialog &dialog)
 {
-  edit_button = dialog.AddButton(_("Edit"), *this, EDIT);
-  flight_button = dialog.AddButton(_("Flight download"), *this, FLIGHT);
-  manage_button = dialog.AddButton(_("Manage"), *this, MANAGE);
-  monitor_button = dialog.AddButton(_("Monitor"), *this, MONITOR);
-  reconnect_button = dialog.AddButton(_("Reconnect"), *this, RECONNECT);
-  disable_button = dialog.AddButton(_("Disable"), *this, DISABLE);
-  debug_button = dialog.AddButton(_("Debug"), *this, DEBUG);
+  edit_button = dialog.AddButton(_("Edit"), [this](){
+    EditCurrent();
+  });
+
+  flight_button = dialog.AddButton(_("Flight download"), [this](){
+    DownloadFlightFromCurrent();
+  });
+
+  manage_button = dialog.AddButton(_("Manage"), [this](){
+    ManageCurrent();
+  });
+
+  monitor_button = dialog.AddButton(_("Monitor"), [this](){
+    MonitorCurrent();
+  });
+
+  reconnect_button = dialog.AddButton(_("Reconnect"), [this](){
+    ReconnectCurrent();
+  });
+
+  disable_button = dialog.AddButton(_("Disable"), [this](){
+    EnableDisableCurrent();
+  });
+
+  debug_button = dialog.AddButton(_("Debug"), [this](){
+    DebugCurrent();
+  });
 }
 
 void
@@ -319,7 +339,8 @@ DeviceListWidget::UpdateButtons()
 }
 
 void
-DeviceListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc, unsigned idx)
+DeviceListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc,
+                              unsigned idx) noexcept
 {
   assert(idx < NUMDEV);
 
@@ -327,7 +348,7 @@ DeviceListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc, unsigned idx)
     CommonInterface::SetSystemSettings().devices[idx];
   const Flags flags(*items[idx]);
 
-  const UPixelScalar margin = Layout::GetTextPadding();
+  const unsigned margin = Layout::GetTextPadding();
 
   TCHAR port_name_buffer[128];
   const TCHAR *port_name =
@@ -345,7 +366,7 @@ DeviceListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc, unsigned idx)
   }
 
   canvas.Select(*look.list.font);
-  canvas.DrawText(rc.left + margin, rc.top + margin, text);
+  canvas.DrawText(rc.GetTopLeft() + PixelSize{margin, margin}, text);
 
   /* show a list of features that are available in the second row */
 
@@ -379,6 +400,11 @@ DeviceListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc, unsigned idx)
     if (flags.traffic)
       buffer.append(_T("; FLARM"));
 
+    if (flags.temperature || flags.humidity) {
+      buffer.append(_T("; "));
+      buffer.append(_T("Environment"));
+    }
+
     if (flags.debug) {
       buffer.append(_T("; "));
       buffer.append(_("Debug"));
@@ -407,18 +433,21 @@ DeviceListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc, unsigned idx)
   } else if (flags.duplicate) {
     status = _("Duplicate");
   } else if (flags.error) {
-    status = _("Error");
+    if (error_messages[idx].empty())
+      status = _("Error");
+    else
+      status = error_messages[idx].c_str();
   } else {
     status = _("Not connected");
   }
 
   canvas.Select(look.small_font);
-  canvas.DrawText(rc.left + margin, rc.top + 2 * margin + font_height,
+  canvas.DrawText(rc.GetTopLeft() + PixelSize{margin, 2 * margin + font_height},
                   status);
 }
 
 void
-DeviceListWidget::OnCursorMoved(unsigned index)
+DeviceListWidget::OnCursorMoved(unsigned index) noexcept
 {
   UpdateButtons();
 }
@@ -579,18 +608,24 @@ DeviceListWidget::ManageCurrent()
   if (descriptor.IsDriver(_T("CAI 302")))
     ManageCAI302Dialog(UIGlobals::GetMainWindow(), look, *device);
   else if (descriptor.IsDriver(_T("FLARM"))) {
-    device_blackboard->mutex.Lock();
-    const NMEAInfo &basic = device_blackboard->RealState(current);
-    const FlarmVersion version = basic.flarm.version;
-    device_blackboard->mutex.Unlock();
+    FlarmVersion version;
+
+    {
+      const std::lock_guard<Mutex> lock(device_blackboard->mutex);
+      const NMEAInfo &basic = device_blackboard->RealState(current);
+      version = basic.flarm.version;
+    }
 
     ManageFlarmDialog(*device, version);
   } else if (descriptor.IsDriver(_T("LX"))) {
-    device_blackboard->mutex.Lock();
-    const NMEAInfo &basic = device_blackboard->RealState(current);
-    const DeviceInfo info = basic.device;
-    const DeviceInfo secondary_info = basic.secondary_device;
-    device_blackboard->mutex.Unlock();
+    DeviceInfo info, secondary_info;
+
+    {
+      const std::lock_guard<Mutex> lock(device_blackboard->mutex);
+      const NMEAInfo &basic = device_blackboard->RealState(current);
+      info = basic.device;
+      secondary_info = basic.secondary_device;
+    }
 
     LXDevice &lx_device = *(LXDevice *)device;
     if (lx_device.IsV7())
@@ -633,7 +668,7 @@ DeviceListWidget::DebugCurrent()
 
   static constexpr unsigned MINUTES = 10;
 
-  device.EnableDumpTemporarily(MINUTES * 60000);
+  device.EnableDumpTemporarily(std::chrono::minutes(MINUTES));
   RefreshList();
 
   StaticString<256> msg;
@@ -642,40 +677,6 @@ DeviceListWidget::DebugCurrent()
   ShowMessageBox(msg, _("Debug"), MB_OK | MB_ICONINFORMATION);
 }
 
-void
-DeviceListWidget::OnAction(int id)
-{
-  switch (id) {
-  case DISABLE:
-    EnableDisableCurrent();
-    break;
-
-  case RECONNECT:
-    ReconnectCurrent();
-    break;
-
-  case FLIGHT:
-    DownloadFlightFromCurrent();
-    break;
-
-  case EDIT:
-    EditCurrent();
-    break;
-
-  case MANAGE:
-    ManageCurrent();
-    break;
-
-  case MONITOR:
-    MonitorCurrent();
-    break;
-
-  case DEBUG:
-    DebugCurrent();
-    break;
-  }
-}
-
 void
 DeviceListWidget::OnGPSUpdate(const MoreData &basic)
 {
@@ -686,14 +687,13 @@ DeviceListWidget::OnGPSUpdate(const MoreData &basic)
 void
 ShowDeviceList()
 {
-  DeviceListWidget widget(UIGlobals::GetDialogLook());
-
-  WidgetDialog dialog(UIGlobals::GetDialogLook());
-  dialog.CreateFull(UIGlobals::GetMainWindow(), _("Devices"), &widget);
-  widget.CreateButtons(dialog);
+  TWidgetDialog<DeviceListWidget>
+    dialog(WidgetDialog::Full{}, UIGlobals::GetMainWindow(),
+           UIGlobals::GetDialogLook(), _("Devices"));
   dialog.AddButton(_("Close"), mrOK);
+  dialog.SetWidget(UIGlobals::GetDialogLook());
+  dialog.GetWidget().CreateButtons(dialog);
   dialog.EnableCursorSelection();
 
   dialog.ShowModal();
-  dialog.StealWidget();
 }
