@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -31,13 +31,13 @@ Copyright_License {
 #include "Language/Language.hpp"
 #include "Tracking/TrackingSettings.hpp"
 #include "Tracking/SkyLines/Key.hpp"
-#include "Net/State.hpp"
+#include "net/State.hpp"
 #include "Form/DataField/Base.hpp"
 #include "Widget/RowFormWidget.hpp"
 #include "Screen/Layout.hpp"
 #include "Interface.hpp"
 #include "UIGlobals.hpp"
-#include "Util/NumberParser.hpp"
+#include "util/NumberParser.hpp"
 
 enum ControlIndex {
 #ifdef HAVE_SKYLINES_TRACKING
@@ -46,24 +46,21 @@ enum ControlIndex {
   SL_ROAMING,
 #endif
   SL_INTERVAL,
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
   SL_TRAFFIC_ENABLED,
   SL_NEAR_TRAFFIC_ENABLED,
-#endif
   SL_KEY,
-  CLOUD_ENABLED,
 #endif
 #if defined(HAVE_SKYLINES_TRACKING) && defined(HAVE_LIVETRACK24)
   SPACER,
 #endif
 #ifdef HAVE_LIVETRACK24
-  LT24Enabled,
-  TrackingInterval,
-  TrackingVehicleType,
-  TrackingVehicleName,
-  LT24Server,
-  LT24Username,
-  LT24Password
+  LT24_ENABLED,
+  LT24_INVERVAL,
+  LT24_VEHICLE_TYPE,
+  LT24_VEHICLE_NAME,
+  LT24_SERVER,
+  LT24_USERNAME,
+  LT24_PASSWORD
 #endif
 };
 
@@ -79,16 +76,16 @@ public:
 #endif
 
 #ifdef HAVE_LIVETRACK24
-  void SetEnabled(bool enabled);
+  void SetLiveTrack24Enabled(bool enabled);
 #endif
 
   /* methods from Widget */
-  virtual void Prepare(ContainerWindow &parent, const PixelRect &rc) override;
-  virtual bool Save(bool &changed) override;
+  void Prepare(ContainerWindow &parent, const PixelRect &rc) noexcept override;
+  bool Save(bool &changed) noexcept override;
 
 private:
   /* methods from DataFieldListener */
-  virtual void OnModified(DataField &df) override;
+  void OnModified(DataField &df) noexcept override;
 };
 
 #ifdef HAVE_SKYLINES_TRACKING
@@ -100,11 +97,9 @@ TrackingConfigPanel::SetSkyLinesEnabled(bool enabled)
   SetRowEnabled(SL_ROAMING, enabled);
 #endif
   SetRowEnabled(SL_INTERVAL, enabled);
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
   SetRowEnabled(SL_TRAFFIC_ENABLED, enabled);
   SetRowEnabled(SL_NEAR_TRAFFIC_ENABLED,
                 enabled && GetValueBoolean(SL_TRAFFIC_ENABLED));
-#endif
   SetRowEnabled(SL_KEY, enabled);
 }
 
@@ -113,20 +108,20 @@ TrackingConfigPanel::SetSkyLinesEnabled(bool enabled)
 #ifdef HAVE_LIVETRACK24
 
 void
-TrackingConfigPanel::SetEnabled(bool enabled)
+TrackingConfigPanel::SetLiveTrack24Enabled(bool enabled)
 {
-  SetRowEnabled(TrackingInterval, enabled);
-  SetRowEnabled(TrackingVehicleType, enabled);
-  SetRowEnabled(TrackingVehicleName, enabled);
-  SetRowEnabled(LT24Server, enabled);
-  SetRowEnabled(LT24Username, enabled);
-  SetRowEnabled(LT24Password, enabled);
+  SetRowEnabled(LT24_INVERVAL, enabled);
+  SetRowEnabled(LT24_VEHICLE_TYPE, enabled);
+  SetRowEnabled(LT24_VEHICLE_NAME, enabled);
+  SetRowEnabled(LT24_SERVER, enabled);
+  SetRowEnabled(LT24_USERNAME, enabled);
+  SetRowEnabled(LT24_PASSWORD, enabled);
 }
 
 #endif
 
 void
-TrackingConfigPanel::OnModified(DataField &df)
+TrackingConfigPanel::OnModified(DataField &df) noexcept
 {
 #ifdef HAVE_SKYLINES_TRACKING
   if (IsDataField(SL_ENABLED, df)) {
@@ -134,9 +129,7 @@ TrackingConfigPanel::OnModified(DataField &df)
     SetSkyLinesEnabled(dfb.GetAsBoolean());
     return;
   }
-#endif
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
   if (IsDataField(SL_TRAFFIC_ENABLED, df)) {
     const DataFieldBoolean &dfb = (const DataFieldBoolean &)df;
     SetRowEnabled(SL_NEAR_TRAFFIC_ENABLED, dfb.GetAsBoolean());
@@ -145,9 +138,9 @@ TrackingConfigPanel::OnModified(DataField &df)
 #endif
 
 #ifdef HAVE_LIVETRACK24
-  if (IsDataField(LT24Enabled, df)) {
+  if (IsDataField(LT24_ENABLED, df)) {
     const DataFieldBoolean &dfb = (const DataFieldBoolean &)df;
-    SetEnabled(dfb.GetAsBoolean());
+    SetLiveTrack24Enabled(dfb.GetAsBoolean());
   }
 #endif
 }
@@ -186,19 +179,19 @@ static constexpr StaticEnumChoice server_list[] = {
 };
 
 static constexpr StaticEnumChoice vehicle_type_list[] = {
-  { (unsigned) TrackingSettings::VehicleType::GLIDER, N_("Glider") },
-  { (unsigned) TrackingSettings::VehicleType::PARAGLIDER, N_("Paraglider") },
-  { (unsigned) TrackingSettings::VehicleType::POWERED_AIRCRAFT, N_("Powered aircraft") },
-  { (unsigned) TrackingSettings::VehicleType::HOT_AIR_BALLOON, N_("Hot-air balloon") },
-  { (unsigned) TrackingSettings::VehicleType::HANGGLIDER_FLEX, N_("Hangglider (Flex/FAI1)") },
-  { (unsigned) TrackingSettings::VehicleType::HANGGLIDER_RIGID, N_("Hangglider (Rigid/FAI5)") },
+  { (unsigned) LiveTrack24::Settings::VehicleType::GLIDER, N_("Glider") },
+  { (unsigned) LiveTrack24::Settings::VehicleType::PARAGLIDER, N_("Paraglider") },
+  { (unsigned) LiveTrack24::Settings::VehicleType::POWERED_AIRCRAFT, N_("Powered aircraft") },
+  { (unsigned) LiveTrack24::Settings::VehicleType::HOT_AIR_BALLOON, N_("Hot-air balloon") },
+  { (unsigned) LiveTrack24::Settings::VehicleType::HANGGLIDER_FLEX, N_("Hangglider (Flex/FAI1)") },
+  { (unsigned) LiveTrack24::Settings::VehicleType::HANGGLIDER_RIGID, N_("Hangglider (Rigid/FAI5)") },
   { 0 },
 };
 
 #endif
 
 void
-TrackingConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
+TrackingConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc) noexcept
 {
   const TrackingSettings &settings =
     CommonInterface::GetComputerSettings().tracking;
@@ -213,7 +206,6 @@ TrackingConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
   AddEnum(_("Tracking Interval"), nullptr, tracking_intervals,
           settings.skylines.interval);
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
   AddBoolean(_("Track friends"),
              _("Download the position of your friends live from the SkyLines server."),
              settings.skylines.traffic_enabled, this);
@@ -221,7 +213,6 @@ TrackingConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
   AddBoolean(_("Show nearby traffic"),
              _("Download the position of your nearby traffic live from the SkyLines server."),
              settings.skylines.near_traffic_enabled, this);
-#endif
 
   StaticString<64> buffer;
   if (settings.skylines.key != 0)
@@ -229,11 +220,6 @@ TrackingConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
   else
     buffer.clear();
   AddText(_T("Key"), nullptr, buffer);
-
-  AddBoolean(_T("XCSoar Cloud"),
-             _("Participate in the XCSoar Cloud field test?  This transmits your location, thermal/wave locations and other weather data to our test server."),
-             settings.skylines.cloud_enabled == TriState::TRUE,
-             this);
 #endif
 
 #if defined(HAVE_SKYLINES_TRACKING) && defined(HAVE_LIVETRACK24)
@@ -243,12 +229,12 @@ TrackingConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
 #ifdef HAVE_LIVETRACK24
   AddBoolean(_T("LiveTrack24"),  _T(""), settings.livetrack24.enabled, this);
 
-  AddTime(_("Tracking Interval"), _T(""), 5, 3600, 5, settings.interval);
+  AddTime(_("Tracking Interval"), _T(""), 5, 3600, 5, settings.livetrack24.interval);
 
   AddEnum(_("Vehicle Type"), _("Type of vehicle used."), vehicle_type_list,
-          (unsigned) settings.vehicleType);
+          (unsigned) settings.livetrack24.vehicleType);
   AddText(_("Vehicle Name"), _T("Name of vehicle used."),
-          settings.vehicle_name);
+          settings.livetrack24.vehicle_name);
 
   WndProperty *edit = AddEnum(_("Server"), _T(""), server_list, 0);
   ((DataFieldEnum *)edit->GetDataField())->Set(settings.livetrack24.server);
@@ -263,7 +249,7 @@ TrackingConfigPanel::Prepare(ContainerWindow &parent, const PixelRect &rc)
 #endif
 
 #ifdef HAVE_LIVETRACK24
-  SetEnabled(settings.livetrack24.enabled);
+  SetLiveTrack24Enabled(settings.livetrack24.enabled);
 #endif
 }
 
@@ -284,7 +270,7 @@ SaveKey(const RowFormWidget &form, unsigned idx, const char *profile_key,
 #endif
 
 bool
-TrackingConfigPanel::Save(bool &_changed)
+TrackingConfigPanel::Save(bool &_changed) noexcept
 {
   bool changed = false;
 
@@ -292,13 +278,13 @@ TrackingConfigPanel::Save(bool &_changed)
     CommonInterface::SetComputerSettings().tracking;
 
 #ifdef HAVE_LIVETRACK24
-  changed |= SaveValue(TrackingInterval, ProfileKeys::TrackingInterval, settings.interval);
+  changed |= SaveValue(LT24_INVERVAL, ProfileKeys::LiveTrack24TrackingInterval, settings.livetrack24.interval);
 
-  changed |= SaveValueEnum(TrackingVehicleType, ProfileKeys::TrackingVehicleType,
-                           settings.vehicleType);
+  changed |= SaveValueEnum(LT24_VEHICLE_TYPE, ProfileKeys::LiveTrack24TrackingVehicleType,
+                           settings.livetrack24.vehicleType);
 
-  changed |= SaveValue(TrackingVehicleName, ProfileKeys::TrackingVehicleName,
-                       settings.vehicle_name);
+  changed |= SaveValue(LT24_VEHICLE_NAME, ProfileKeys::LiveTrack24TrackingVehicleName,
+                       settings.livetrack24.vehicle_name);
 #endif
 
 #ifdef HAVE_SKYLINES_TRACKING
@@ -313,47 +299,26 @@ TrackingConfigPanel::Save(bool &_changed)
   changed |= SaveValue(SL_INTERVAL, ProfileKeys::SkyLinesTrackingInterval,
                        settings.skylines.interval);
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
   changed |= SaveValue(SL_TRAFFIC_ENABLED, ProfileKeys::SkyLinesTrafficEnabled,
                        settings.skylines.traffic_enabled);
   changed |= SaveValue(SL_NEAR_TRAFFIC_ENABLED,
                        ProfileKeys::SkyLinesNearTrafficEnabled,
                        settings.skylines.near_traffic_enabled);
-#endif
 
   changed |= SaveKey(*this, SL_KEY, ProfileKeys::SkyLinesTrackingKey,
                      settings.skylines.key);
-
-  bool cloud_enabled = settings.skylines.cloud_enabled == TriState::TRUE;
-  if (SaveValue(CLOUD_ENABLED, ProfileKeys::CloudEnabled, cloud_enabled)) {
-    settings.skylines.cloud_enabled = cloud_enabled
-      ? TriState::TRUE
-      : TriState::FALSE;
-
-    if (settings.skylines.cloud_enabled == TriState::TRUE &&
-        settings.skylines.cloud_key == 0) {
-      settings.skylines.cloud_key = SkyLinesTracking::GenerateKey();
-
-      char s[64];
-      snprintf(s, sizeof(s), "%llx",
-               (unsigned long long)settings.skylines.cloud_key);
-      Profile::Set(ProfileKeys::CloudKey, s);
-    }
-
-    changed = true;
-  }
 #endif
 
 #ifdef HAVE_LIVETRACK24
-  changed |= SaveValue(LT24Enabled, ProfileKeys::LiveTrack24Enabled, settings.livetrack24.enabled);
+  changed |= SaveValue(LT24_ENABLED, ProfileKeys::LiveTrack24Enabled, settings.livetrack24.enabled);
 
-  changed |= SaveValue(LT24Server, ProfileKeys::LiveTrack24Server,
+  changed |= SaveValue(LT24_SERVER, ProfileKeys::LiveTrack24Server,
                        settings.livetrack24.server);
 
-  changed |= SaveValue(LT24Username, ProfileKeys::LiveTrack24Username,
+  changed |= SaveValue(LT24_USERNAME, ProfileKeys::LiveTrack24Username,
                        settings.livetrack24.username);
 
-  changed |= SaveValue(LT24Password, ProfileKeys::LiveTrack24Password,
+  changed |= SaveValue(LT24_PASSWORD, ProfileKeys::LiveTrack24Password,
                        settings.livetrack24.password);
 #endif
 
@@ -362,8 +327,8 @@ TrackingConfigPanel::Save(bool &_changed)
   return true;
 }
 
-Widget *
+std::unique_ptr<Widget>
 CreateTrackingConfigPanel()
 {
-  return new TrackingConfigPanel();
+  return std::make_unique<TrackingConfigPanel>();
 }
