@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,15 +22,15 @@ Copyright_License {
 */
 
 #include "StartupDialog.hpp"
+#include "Error.hpp"
 #include "ProfilePasswordDialog.hpp"
 #include "ProfileListDialog.hpp"
-#include "Message.hpp"
 #include "WidgetDialog.hpp"
 #include "Widget/TwoWidgets.hpp"
 #include "Widget/RowFormWidget.hpp"
 #include "UIGlobals.hpp"
 #include "Profile/Profile.hpp"
-#include "Screen/Canvas.hpp"
+#include "ui/canvas/Canvas.hpp"
 #include "Screen/Layout.hpp"
 #include "Look/DialogLook.hpp"
 #include "Form/Form.hpp"
@@ -39,13 +39,8 @@ Copyright_License {
 #include "Language/Language.hpp"
 #include "Gauge/LogoView.hpp"
 #include "LogFile.hpp"
-#include "Util/StringUtil.hpp"
 #include "LocalPath.hpp"
-#include "OS/PathName.hpp"
-#include "OS/FileUtil.hpp"
-#include "Compiler.h"
-
-#include <windef.h> /* for MAX_PATH */
+#include "system/FileUtil.hpp"
 
 class LogoWindow final : public PaintWindow {
   LogoView logo;
@@ -59,14 +54,14 @@ protected:
 
 class LogoQuitWidget final : public NullWidget {
   const ButtonLook &look;
-  ActionListener &action_listener;
+  WndForm &dialog;
 
   LogoWindow logo;
   Button quit;
 
 public:
-  LogoQuitWidget(const ButtonLook &_look, ActionListener &_action_listener)
-    :look(_look), action_listener(_action_listener) {}
+  LogoQuitWidget(const ButtonLook &_look, WndForm &_dialog) noexcept
+    :look(_look), dialog(_dialog) {}
 
 private:
   PixelRect GetButtonRect(PixelRect rc) {
@@ -77,17 +72,17 @@ private:
 
 public:
   /* virtual methods from class Widget */
-  virtual PixelSize GetMinimumSize() const override {
+  PixelSize GetMinimumSize() const noexcept override {
     return { 150, 150 };
   }
 
-  virtual PixelSize GetMaximumSize() const override {
+  PixelSize GetMaximumSize() const noexcept override {
     /* use as much as possible */
     return { 8192, 8192 };
   }
 
-  virtual void Prepare(ContainerWindow &parent,
-                       const PixelRect &rc) override {
+  void Prepare(ContainerWindow &parent,
+               const PixelRect &rc) noexcept override {
     WindowStyle style;
     style.Hide();
 
@@ -96,26 +91,21 @@ public:
     button_style.TabStop();
 
     quit.Create(parent, look, _("Quit"), rc,
-                button_style, action_listener, mrCancel);
+                button_style, dialog.MakeModalResultCallback(mrCancel));
     logo.Create(parent, rc, style);
   }
 
-  virtual void Unprepare() override {
-    logo.Destroy();
-    quit.Destroy();
-  }
-
-  virtual void Show(const PixelRect &rc) override {
+  void Show(const PixelRect &rc) noexcept override {
     quit.MoveAndShow(GetButtonRect(rc));
     logo.MoveAndShow(rc);
   }
 
-  virtual void Hide() override {
+  void Hide() noexcept override {
     quit.FastHide();
     logo.FastHide();
   }
 
-  virtual void Move(const PixelRect &rc) override {
+  void Move(const PixelRect &rc) noexcept override {
     quit.Move(GetButtonRect(rc));
     logo.Move(rc);
   }
@@ -127,20 +117,20 @@ class StartupWidget final : public RowFormWidget {
     CONTINUE,
   };
 
-  ActionListener &action_listener;
+  WndForm &dialog;
   DataField *const df;
 
 public:
-  StartupWidget(const DialogLook &look, ActionListener &_action_listener,
+  StartupWidget(const DialogLook &look, WndForm &_dialog,
                 DataField *_df)
-    :RowFormWidget(look), action_listener(_action_listener), df(_df) {}
+    :RowFormWidget(look), dialog(_dialog), df(_df) {}
 
   /* virtual methods from class Widget */
-  virtual void Prepare(ContainerWindow &parent,
-                       const PixelRect &rc) override;
-  virtual bool Save(bool &changed) override;
+  void Prepare(ContainerWindow &parent,
+               const PixelRect &rc) noexcept override;
+  bool Save(bool &changed) noexcept override;
 
-  virtual bool SetFocus() override {
+  bool SetFocus() noexcept override {
     /* focus the "Continue" button by default */
     GetRow(CONTINUE).SetFocus();
     return true;
@@ -154,47 +144,47 @@ SelectProfileCallback(const TCHAR *caption, DataField &_df,
   FileDataField &df = (FileDataField &)_df;
 
   const auto path = SelectProfileDialog(df.GetPathFile());
-  if (path.empty())
+  if (path.IsNull())
     return false;
 
-  df.ForceModify(path.c_str());
+  df.ForceModify(path);
   return true;
 }
 
 void
 StartupWidget::Prepare(ContainerWindow &parent,
-                       const PixelRect &rc)
+                       const PixelRect &rc) noexcept
 {
   auto *pe = Add(_("Profile"), nullptr, df);
   pe->SetEditCallback(SelectProfileCallback);
 
-  AddButton(_("Continue"), action_listener, mrOK);
+  AddButton(_("Continue"), dialog.MakeModalResultCallback(mrOK));
 }
 
 static bool
-SelectProfile(const TCHAR *path)
+SelectProfile(Path path)
 {
-  if (StringIsEmpty(path))
-    return true;
-
-  if (!CheckProfilePasswordResult(CheckProfileFilePassword(path)))
+  try {
+    if (!CheckProfilePasswordResult(CheckProfileFilePassword(path)))
+      return false;
+  } catch (...) {
+    ShowError(std::current_exception(), _("Password"));
     return false;
+  }
 
   Profile::SetFiles(path);
 
-  if (RelativePath(path) == nullptr) {
+  if (RelativePath(path) == nullptr)
     /* When a profile from a secondary data path is used, this path
        becomes the primary data path */
-    TCHAR temp[MAX_PATH];
-    SetPrimaryDataPath(DirName(path, temp));
-  }
+    SetPrimaryDataPath(path.GetParent());
 
   File::Touch(path);
   return true;
 }
 
 bool
-StartupWidget::Save(bool &changed)
+StartupWidget::Save(bool &changed) noexcept
 {
   const auto &dff = (const FileDataField &)GetDataField(PROFILE);
   if (!SelectProfile(dff.GetPathFile()))
@@ -214,14 +204,18 @@ dlgStartupShowModal()
   auto *dff = new FileDataField();
   dff->ScanDirectoryTop(_T("*.prf"));
 
-  /* skip this dialog if there is only one (or none) */
-  if (dff->GetNumFiles() <= 1) {
-    const TCHAR *path = dff->GetPathFile();
-    if (!ProfileFileHasPassword(path)) {
-      SelectProfile(path);
+  if (dff->GetNumFiles() == 1) {
+    /* skip this dialog if there is only one */
+    const auto path = dff->GetPathFile();
+    if (ProfileFileHasPassword(path) == TriState::FALSE &&
+        SelectProfile(path)) {
       delete dff;
       return true;
     }
+  } else if (dff->GetNumFiles() == 0) {
+    /* no profile exists yet: create default profile */
+    Profile::SetFiles(nullptr);
+    return true;
   }
 
   /* preselect the most recently used profile */
@@ -230,7 +224,7 @@ dlgStartupShowModal()
   unsigned length = dff->size();
 
   for (unsigned i = 0; i < length; ++i) {
-    const TCHAR *path = dff->GetItem(i);
+    const auto path = dff->GetItem(i);
     uint64_t timestamp = File::GetLastModification(path);
     if (timestamp > best_timestamp) {
       best_timestamp = timestamp;
@@ -242,14 +236,13 @@ dlgStartupShowModal()
 
   /* show the dialog */
   const DialogLook &look = UIGlobals::GetDialogLook();
-  WidgetDialog dialog(look);
-  TwoWidgets widget(new LogoQuitWidget(look.button, dialog),
-                    new StartupWidget(look, dialog, dff));
+  TWidgetDialog<TwoWidgets> dialog(WidgetDialog::Full{},
+                                   UIGlobals::GetMainWindow(),
+                                   UIGlobals::GetDialogLook(),
+                                   nullptr);
 
-  dialog.CreateFull(UIGlobals::GetMainWindow(), _T(""), &widget);
+  dialog.SetWidget(std::make_unique<LogoQuitWidget>(look.button, dialog),
+                   std::make_unique<StartupWidget>(look, dialog, dff));
 
-  const int result = dialog.ShowModal();
-  dialog.StealWidget();
-
-  return result == mrOK;
+  return dialog.ShowModal() == mrOK;
 }
