@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -32,11 +32,12 @@ class OrderedTask;
 class TaskMiscPanel final : public PagerWidget {
 public:
   TaskMiscPanel(TaskManagerDialog &dialog,
-                OrderedTask **_active_task, bool *_task_modified);
+                std::unique_ptr<OrderedTask> &_active_task,
+                bool *_task_modified) noexcept;
 
   /* virtual methods from class Widget */
-  void ReClick() override;
-  void Show(const PixelRect &rc) override;
+  void ReClick() noexcept override;
+  void Show(const PixelRect &rc) noexcept override;
 };
 
 #endif
