@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,10 +24,10 @@
 #include "Task/Stats/DistanceStat.hpp"
 
 void
-DistanceStatComputer::CalcSpeed(DistanceStat &data, fixed time)
+DistanceStatComputer::CalcSpeed(DistanceStat &data, double time)
 {
-  if (positive(time) && data.IsDefined())
+  if (time > 0 && data.IsDefined())
     data.speed = data.GetDistance() / time;
   else
-    data.speed = fixed(0);
+    data.speed = 0;
 }
