@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,19 +24,13 @@ Copyright_License {
 #include "File.hpp"
 #include "ComboList.hpp"
 #include "LocalPath.hpp"
-#include "Util/StringUtil.hpp"
-#include "Util/StringAPI.hxx"
-#include "OS/PathName.hpp"
-#include "OS/FileUtil.hpp"
-
-#if defined(_WIN32_WCE) && !defined(GNAV)
-#include "OS/FlashCardEnumerator.hpp"
-#endif
+#include "util/StringAPI.hxx"
+#include "system/FileUtil.hpp"
 
 #include <algorithm>
 
 #include <windef.h> /* for MAX_PATH */
-#include <assert.h>
+#include <cassert>
 #include <stdlib.h>
 
 /**
@@ -46,7 +40,7 @@ Copyright_License {
  */
 gcc_pure
 static bool
-IsInternalFile(const TCHAR* str)
+IsInternalFile(const TCHAR *str) noexcept
 {
   static const TCHAR *const ifiles[] = {
     _T("xcsoar-checklist.txt"),
@@ -73,59 +67,52 @@ private:
   FileDataField &datafield;
 
 public:
-  FileVisitor(FileDataField &_datafield) : datafield(_datafield) {}
+  explicit FileVisitor(FileDataField &_datafield) noexcept
+    : datafield(_datafield) {}
 
-  void
-  Visit(const TCHAR* path, const TCHAR* filename)
-  {
-    if (!IsInternalFile(filename))
-      datafield.AddFile(filename, path);
+  void Visit(Path path, Path filename) override {
+    if (!IsInternalFile(filename.c_str()))
+      datafield.AddFile(path);
   }
 };
 
-FileDataField::Item::~Item()
-{
-  free(path);
-}
-
 inline void
-FileDataField::Item::Set(const TCHAR *_path)
+FileDataField::Item::Set(Path _path) noexcept
 {
-  free(path);
-  path = _tcsdup(_path);
-  filename = BaseName(path);
+  path = _path;
+  filename = path.GetBase();
   if (filename == nullptr)
     filename = path;
 }
 
-FileDataField::FileDataField(DataFieldListener *listener)
+FileDataField::FileDataField(DataFieldListener *listener) noexcept
   :DataField(Type::FILE, true, listener),
    // Set selection to zero
    current_index(0),
    loaded(false), postponed_sort(false),
-   postponed_value(_T("")) {}
+   postponed_value(nullptr) {}
 
 int
-FileDataField::GetAsInteger() const
+FileDataField::GetAsInteger() const noexcept
 {
-  if (!postponed_value.empty())
+  if (!postponed_value.IsNull())
     EnsureLoadedDeconst();
 
   return current_index;
 }
 
 void
-FileDataField::SetAsInteger(int new_value)
+FileDataField::SetAsInteger(int new_value) noexcept
 {
   Set(new_value);
 }
 
 void
-FileDataField::ScanDirectoryTop(const TCHAR *filter)
+FileDataField::ScanDirectoryTop(const TCHAR *filter) noexcept
 {
   if (!loaded) {
     if (!postponed_patterns.full() &&
-        _tcslen(filter) < PatternList::value_type::CAPACITY) {
+        _tcslen(filter) < PatternList::value_type().capacity()) {
       postponed_patterns.append() = filter;
       return;
     } else
@@ -139,7 +126,7 @@ FileDataField::ScanDirectoryTop(const TCHAR *filter)
 }
 
 void
-FileDataField::ScanMultiplePatterns(const TCHAR *patterns)
+FileDataField::ScanMultiplePatterns(const TCHAR *patterns) noexcept
 {
   size_t length;
   while ((length = _tcslen(patterns)) > 0) {
@@ -149,24 +136,21 @@ FileDataField::ScanMultiplePatterns(const TCHAR *patterns)
 }
 
 int
-FileDataField::Find(const TCHAR *text) const
+FileDataField::Find(Path path) const noexcept
 {
   for (unsigned i = 0, n = files.size(); i < n; i++)
-    if (StringIsEqual(text, files[i].path))
+    if (files[i].path == path)
       return i;
 
   return -1;
 }
 
 void
-FileDataField::Lookup(const TCHAR *text)
+FileDataField::Lookup(Path text) noexcept
 {
   if (!loaded) {
-    if (_tcslen(text) < postponed_value.capacity()) {
-      postponed_value = text;
-      return;
-    } else
-      EnsureLoaded();
+    postponed_value = text;
+    return;
   }
 
   auto i = Find(text);
@@ -175,7 +159,7 @@ FileDataField::Lookup(const TCHAR *text)
 }
 
 void
-FileDataField::ForceModify(const TCHAR *path)
+FileDataField::ForceModify(Path path) noexcept
 {
   EnsureLoaded();
 
@@ -194,29 +178,30 @@ FileDataField::ForceModify(const TCHAR *path)
 }
 
 unsigned
-FileDataField::GetNumFiles() const
+FileDataField::GetNumFiles() const noexcept
 {
   EnsureLoadedDeconst();
 
   return files.size();
 }
 
-const TCHAR *
-FileDataField::GetPathFile() const
+Path
+FileDataField::GetPathFile() const noexcept
 {
-  if (!loaded)
+  if (!loaded && postponed_value != nullptr)
     return postponed_value;
 
   if (current_index >= files.size())
-    return _T("");
+    // TODO: return nullptr instead of empty string?
+    return Path(_T(""));
 
-  const TCHAR *path = files[current_index].path;
+  const Path path = files[current_index].path;
   assert(path != nullptr);
   return path;
 }
 
 void
-FileDataField::AddFile(const TCHAR *filename, const TCHAR *path)
+FileDataField::AddFile(Path path) noexcept
 {
   assert(loaded);
 
@@ -231,52 +216,52 @@ FileDataField::AddFile(const TCHAR *filename, const TCHAR *path)
 }
 
 void
-FileDataField::AddNull()
+FileDataField::AddNull() noexcept
 {
   assert(!files.full());
 
   Item &item = files.append();
-  item.filename = _T("");
-  item.path = _tcsdup(_T(""));
+  item.filename = Path(_T(""));
+  item.path = Path(_T(""));
 }
 
 const TCHAR *
-FileDataField::GetAsString() const
+FileDataField::GetAsString() const noexcept
 {
-  if (!loaded)
-    return postponed_value;
+  if (!loaded && postponed_value != nullptr)
+    return postponed_value.c_str();
 
   if (current_index < files.size())
-    return files[current_index].path;
+    return files[current_index].path.c_str();
   else
     return _T("");
 }
 
 const TCHAR *
-FileDataField::GetAsDisplayString() const
+FileDataField::GetAsDisplayString() const noexcept
 {
-  if (!loaded) {
+  if (!loaded && postponed_value != nullptr) {
     /* get basename from postponed_value */
-    const TCHAR *p = BaseName(postponed_value);
+    auto p = postponed_value.GetBase();
     if (p == nullptr)
       p = postponed_value;
 
-    return p;
+    return p.c_str();
   }
 
   if (current_index < files.size())
-    return files[current_index].filename;
+    return files[current_index].filename.c_str();
   else
     return _T("");
 }
 
 void
-FileDataField::Set(unsigned new_value)
+FileDataField::Set(unsigned new_value) noexcept
 {
   if (new_value > 0)
     EnsureLoaded();
   else
-    postponed_value.clear();
+    postponed_value = nullptr;
 
   if (new_value < files.size()) {
     current_index = new_value;
@@ -285,7 +270,7 @@ FileDataField::Set(unsigned new_value)
 }
 
 void
-FileDataField::Inc()
+FileDataField::Inc() noexcept
 {
   EnsureLoaded();
 
@@ -296,7 +281,7 @@ FileDataField::Inc()
 }
 
 void
-FileDataField::Dec()
+FileDataField::Dec() noexcept
 {
   if (current_index > 0) {
     current_index--;
@@ -305,7 +290,7 @@ FileDataField::Dec()
 }
 
 void
-FileDataField::Sort()
+FileDataField::Sort() noexcept
 {
   if (!loaded) {
     postponed_sort = true;
@@ -316,12 +301,12 @@ FileDataField::Sort()
   std::sort(files.begin(), files.end(), [](const Item &a,
                                            const Item &b) {
               // Compare by filename
-              return StringCollate(a.filename, b.filename) < 0;
+              return StringCollate(a.filename.c_str(), b.filename.c_str()) < 0;
             });
 }
 
 ComboList
-FileDataField::CreateComboList(const TCHAR *reference) const
+FileDataField::CreateComboList(const TCHAR *reference) const noexcept
 {
   /* sorry for the const_cast .. this method keeps the promise of not
      modifying the object, given that one does not count filling the
@@ -333,7 +318,7 @@ FileDataField::CreateComboList(const TCHAR *reference) const
   TCHAR buffer[MAX_PATH];
 
   for (unsigned i = 0; i < files.size(); i++) {
-    const TCHAR *path = files[i].filename;
+    const Path path = files[i].filename;
     assert(path != nullptr);
 
     /* is a file with the same base name present in another data
@@ -341,23 +326,24 @@ FileDataField::CreateComboList(const TCHAR *reference) const
 
     bool found = false;
     for (unsigned j = 1; j < files.size(); j++) {
-      if (j != i && StringIsEqual(path, files[j].filename)) {
+      if (j != i && files[j].filename == path) {
         found = true;
         break;
       }
     }
 
+    const TCHAR *display_string = path.c_str();
     if (found) {
       /* yes - append the absolute path to allow the user to see the
          difference */
-      _tcscpy(buffer, path);
+      _tcscpy(buffer, path.c_str());
       _tcscat(buffer, _T(" ("));
-      _tcscat(buffer, files[i].path);
+      _tcscat(buffer, files[i].path.c_str());
       _tcscat(buffer, _T(")"));
-      path = buffer;
+      display_string = buffer;
     }
 
-    combo_list.Append(i, path);
+    combo_list.Append(display_string);
   }
 
   combo_list.current_index = current_index;
@@ -366,15 +352,15 @@ FileDataField::CreateComboList(const TCHAR *reference) const
 }
 
 unsigned
-FileDataField::size() const
+FileDataField::size() const noexcept
 {
   EnsureLoadedDeconst();
 
   return files.size();
 }
 
-const TCHAR *
-FileDataField::GetItem(unsigned index) const
+Path
+FileDataField::GetItem(unsigned index) const noexcept
 {
   EnsureLoadedDeconst();
 
@@ -382,7 +368,7 @@ FileDataField::GetItem(unsigned index) const
 }
 
 void
-FileDataField::EnsureLoaded()
+FileDataField::EnsureLoaded() noexcept
 {
   if (loaded)
     return;
@@ -396,6 +382,6 @@ FileDataField::EnsureLoaded()
   if (postponed_sort)
     Sort();
 
-  if (!StringIsEmpty(postponed_value))
+  if (!postponed_value.IsNull())
     Lookup(postponed_value);
 }
