@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -26,8 +26,6 @@ Copyright_License {
 #define XCSOAR_GeoPoint_HPP
 
 #include "Math/Angle.hpp"
-#include "Rough/RoughAltitude.hpp"
-#include "Compiler.h"
 
 #include <type_traits>
 
@@ -136,8 +134,8 @@ struct GeoPoint {
    *
    * @return Location of point
    */
-  gcc_pure
-  GeoPoint Parametric(const GeoPoint &delta, const fixed t) const;
+  [[gnu::pure]]
+  GeoPoint Parametric(const GeoPoint &delta, double t) const;
 
   /**
    * Find location interpolated from this point towards end
@@ -147,8 +145,8 @@ struct GeoPoint {
    *
    * @return Location of point
    */
-  gcc_pure
-  GeoPoint Interpolate(const GeoPoint &end, const fixed t) const;
+  [[gnu::pure]]
+  GeoPoint Interpolate(const GeoPoint &end, double t) const;
 
   /**
    * Multiply a point by a factor (used for deltas)
@@ -157,8 +155,8 @@ struct GeoPoint {
    *
    * @return Modified point
    */
-  gcc_pure
-  GeoPoint operator* (const fixed x) const {
+  [[gnu::pure]]
+  GeoPoint operator* (const double x) const {
     GeoPoint res = *this;
     res.longitude *= x;
     res.latitude *= x;
@@ -172,7 +170,7 @@ struct GeoPoint {
    *
    * @return Modified point
    */
-  gcc_pure
+  [[gnu::pure]]
   GeoPoint operator+ (const GeoPoint &delta) const {
     GeoPoint res = *this;
     res.longitude += delta.longitude;
@@ -200,7 +198,7 @@ struct GeoPoint {
    *
    * @return Modified point
    */
-  gcc_pure
+  [[gnu::pure]]
   GeoPoint operator- (const GeoPoint &delta) const {
     GeoPoint res = *this;
     res.longitude -= delta.longitude;
@@ -215,8 +213,8 @@ struct GeoPoint {
    *
    * @return Distance (m)
    */
-  gcc_pure
-  fixed Distance(const GeoPoint &other) const;
+  [[gnu::pure]]
+  double Distance(const GeoPoint &other) const;
 
   /**
    * Calculate great circle initial bearing from this to the other
@@ -225,21 +223,35 @@ struct GeoPoint {
    *
    * @return Bearing (deg)
    */
-  gcc_pure
+  [[gnu::pure]]
   Angle Bearing(const GeoPoint &other) const;
 
   /**
    * Calculate great circle distance and initial bearing from this to the other
    */
-  gcc_pure
+  [[gnu::pure]]
   GeoVector DistanceBearing(const GeoPoint &other) const;
 
   /**
    * Like Distance(), but use a simplified faster formula that may be
    * less accurate.
    */
-  gcc_pure
-  fixed DistanceS(const GeoPoint &other) const;
+  [[gnu::pure]]
+  double DistanceS(const GeoPoint &other) const;
+
+  /**
+   * Like Bearing(), but use a simplified faster formula that may be
+   * less accurate.
+   */
+  [[gnu::pure]]
+  Angle BearingS(const GeoPoint &other) const;
+
+  /**
+   * Like DistanceBearing(), but use a simplified faster formula that
+   * may be less accurate.
+   */
+  [[gnu::pure]]
+  GeoVector DistanceBearingS(const GeoPoint &other) const;
 
   /**
    * Find distance along a great-circle path that this point
@@ -250,8 +262,8 @@ struct GeoPoint {
    *
    * @return Distance (m) along from-to line
    */
-  gcc_pure
-  fixed ProjectedDistance(const GeoPoint &from, const GeoPoint &to) const;
+  [[gnu::pure]]
+  double ProjectedDistance(const GeoPoint &from, const GeoPoint &to) const;
 
   /**
    * Find point a set distance along a great-circle path towards
@@ -262,15 +274,15 @@ struct GeoPoint {
    *
    * @return Location of point
    */
-  gcc_pure
+  [[gnu::pure]]
   GeoPoint IntermediatePoint(const GeoPoint &destination,
-                             const fixed distance) const;
+                             double distance) const;
 
   /**
    * Find the nearest great-circle middle point between this point and
    * the specified one.
    */
-  gcc_pure
+  [[gnu::pure]]
   GeoPoint Middle(const GeoPoint &other) const;
 
   /**
@@ -316,7 +328,7 @@ struct GeoPoint {
    *
    * @return True if this point is further left (or if equal, lower) than the other
    */
-  gcc_pure
+  [[gnu::pure]]
   bool Sort(const GeoPoint &other) const;
 };
 
@@ -327,12 +339,12 @@ static_assert(std::is_trivial<GeoPoint>::value, "type is not trivial");
  */
 struct AGeoPoint: public GeoPoint {
   /**< Nav reference altitude (m) */
-  RoughAltitude altitude;
+  double altitude;
 
   AGeoPoint() = default;
 
   constexpr
-  AGeoPoint(const GeoPoint p, const RoughAltitude alt)
+  AGeoPoint(const GeoPoint p, const double alt)
     :GeoPoint(p),altitude(alt) {};
 };
 
