@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -47,61 +47,70 @@ doc/html/advanced/input/ALL		http://xcsoar.sourceforge.net/advanced/input/
 #include "InputEvents.hpp"
 #include "InputConfig.hpp"
 #include "InputParser.hpp"
-#include "UIActions.hpp"
 #include "Interface.hpp"
 #include "MainWindow.hpp"
 #include "Protection.hpp"
 #include "LogFile.hpp"
 #include "Menu/ButtonLabel.hpp"
 #include "Profile/ProfileKeys.hpp"
-#include "Asset.hpp"
 #include "Menu/MenuData.hpp"
-#include "IO/ConfiguredFile.hpp"
-#include "IO/LineReader.hpp"
-#include "InfoBoxes/InfoBoxManager.hpp"
+#include "io/ConfiguredFile.hpp"
+#include "io/LineReader.hpp"
 #include "Pan.hpp"
+#include "Dialogs/LockScreen.hpp"
+#include "Menu/MenuBar.hpp"
+#include "MapWindow/GlueMapWindow.hpp"
 
 #ifdef KOBO
-#include "Screen/Key.h"
+#include "ui/event/KeyCode.hpp"
 #endif
 
-#include <assert.h>
+#include "lua/InputEvent.hpp"
+
+#include <cassert>
 #include <tchar.h>
 #include <stdio.h>
 #include <memory>
 
-namespace InputEvents
-{
-  static const TCHAR *flavour;
+namespace InputEvents {
 
-  static Mode current_mode = InputEvents::MODE_DEFAULT;
+static const TCHAR *flavour;
 
-  /**
-   * A mode that overrides the #current_mode.  Only if a value does
-   * not exist in this mode, it will be taken from the #current_mode.
-   * The special value #MODE_DEFAULT means there is no overlay mode.
-   */
-  static Mode overlay_mode = MODE_DEFAULT;
+static Mode current_mode = InputEvents::MODE_DEFAULT;
 
-  static unsigned MenuTimeOut = 0;
+/**
+ * A mode that overrides the #current_mode.  Only if a value does
+ * not exist in this mode, it will be taken from the #current_mode.
+ * The special value #MODE_DEFAULT means there is no overlay mode.
+ */
+static Mode overlay_mode = MODE_DEFAULT;
 
-  gcc_pure
-  static Mode getModeID();
+static unsigned MenuTimeOut = 0;
 
-  static void UpdateOverlayMode();
+gcc_pure
+static Mode
+getModeID() noexcept;
 
-  gcc_pure
-  static unsigned gesture_to_event(const TCHAR *data);
+static void
+UpdateOverlayMode() noexcept;
 
-  /**
-   * @param full if false, update only the dynamic labels
-   */
-  static void drawButtons(Mode mode, bool full=false);
+gcc_pure
+static unsigned
+gesture_to_event(const TCHAR *data) noexcept;
 
-  static void ProcessMenuTimer();
+/**
+ * @param full if false, update only the dynamic labels
+ */
+static void
+drawButtons(Mode mode, bool full=false) noexcept;
 
-  static void processGo(unsigned event_id);
-};
+static void
+ProcessMenuTimer() noexcept;
+
+static void
+processGo(unsigned event_id) noexcept;
+
+} // namespace InputEvents
 
 static InputConfig input_config;
 
@@ -117,13 +126,13 @@ InputEvents::readFile()
   LoadDefaults(input_config);
 
   // Read in user defined configuration file
-  std::unique_ptr<TLineReader> reader(OpenConfiguredTextFile(ProfileKeys::InputFile));
+  auto reader = OpenConfiguredTextFile(ProfileKeys::InputFile);
   if (reader)
     ::ParseInputFile(input_config, *reader);
 }
 
 void
-InputEvents::setMode(Mode mode)
+InputEvents::setMode(Mode mode) noexcept
 {
   assert((unsigned)mode < input_config.modes.size());
 
@@ -137,7 +146,7 @@ InputEvents::setMode(Mode mode)
 }
 
 void
-InputEvents::setMode(const TCHAR *mode)
+InputEvents::setMode(const TCHAR *mode) noexcept
 {
   int m = input_config.LookupMode(mode);
   if (m >= 0)
@@ -145,13 +154,13 @@ InputEvents::setMode(const TCHAR *mode)
 }
 
 void
-InputEvents::UpdatePan()
+InputEvents::UpdatePan() noexcept
 {
   drawButtons(getModeID(), true);
 }
 
 void
-InputEvents::SetFlavour(const TCHAR *_flavour)
+InputEvents::SetFlavour(const TCHAR *_flavour) noexcept
 {
   if (flavour == NULL && _flavour == NULL)
     /* optimised default case */
@@ -168,7 +177,7 @@ InputEvents::SetFlavour(const TCHAR *_flavour)
 }
 
 bool
-InputEvents::IsFlavour(const TCHAR *_flavour)
+InputEvents::IsFlavour(const TCHAR *_flavour) noexcept
 {
   if (flavour == NULL)
     return _flavour == NULL;
@@ -180,14 +189,13 @@ InputEvents::IsFlavour(const TCHAR *_flavour)
 }
 
 bool
-InputEvents::IsDefault()
+InputEvents::IsDefault() noexcept
 {
   return current_mode == MODE_DEFAULT;
 }
 
-
 void
-InputEvents::drawButtons(Mode mode, bool full)
+InputEvents::drawButtons(Mode mode, bool full) noexcept
 {
   if (!global_running)
     return;
@@ -198,10 +206,20 @@ InputEvents::drawButtons(Mode mode, bool full)
     : NULL;
 
   ButtonLabel::Set(menu, overlay_menu, full);
+
+  GlueMapWindow *map = CommonInterface::main_window->GetMapIfActive();
+  if (map != nullptr){
+      if (mode != MODE_DEFAULT){
+          // Set margin so that GlueMapWindow doesn't draw HUD underneath buttons
+          map->SetBottomMarginFactor(menubar_height_scale_factor);
+      } else {
+          map->SetBottomMarginFactor(0);
+      }
+  }
 }
 
 InputEvents::Mode
-InputEvents::getModeID()
+InputEvents::getModeID() noexcept
 {
   if (current_mode == MODE_DEFAULT && IsPanning())
     return MODE_PAN;
@@ -210,7 +228,7 @@ InputEvents::getModeID()
 }
 
 void
-InputEvents::UpdateOverlayMode()
+InputEvents::UpdateOverlayMode() noexcept
 {
   if (flavour != NULL) {
     /* build the "flavoured" mode name from the current "major" mode
@@ -220,7 +238,7 @@ InputEvents::UpdateOverlayMode()
                 flavour);
 
     /* see if it exists */
-    int new_mode = input_config.LookupMode(name);
+    int new_mode = input_config.LookupMode(name.c_str());
     if (new_mode >= 0)
       /* yep, it does */
       overlay_mode = (Mode)new_mode;
@@ -239,7 +257,7 @@ InputEvents::UpdateOverlayMode()
 gcc_pure
 static int
 FindMenuItemByEvent(InputEvents::Mode mode, InputEvents::Mode overlay_mode,
-                    unsigned event_id)
+                    unsigned event_id) noexcept
 {
   const Menu *const overlay_menu = overlay_mode != InputEvents::MODE_DEFAULT
     ? &input_config.menus[overlay_mode]
@@ -260,7 +278,7 @@ FindMenuItemByEvent(InputEvents::Mode mode, InputEvents::Mode overlay_mode,
 }
 
 void
-InputEvents::ProcessEvent(unsigned event_id)
+InputEvents::ProcessEvent(unsigned event_id) noexcept
 {
   assert(event_id != 0);
 
@@ -281,7 +299,7 @@ InputEvents::ProcessEvent(unsigned event_id)
  */
 gcc_pure
 static unsigned
-key_to_event(InputEvents::Mode mode, unsigned key_code)
+key_to_event(InputEvents::Mode mode, unsigned key_code) noexcept
 {
   return input_config.GetKeyEvent(mode, key_code);
 }
@@ -289,7 +307,7 @@ key_to_event(InputEvents::Mode mode, unsigned key_code)
 gcc_pure
 static unsigned
 key_to_event(InputEvents::Mode mode, InputEvents::Mode overlay_mode,
-             unsigned key_code)
+             unsigned key_code) noexcept
 {
   if (overlay_mode != InputEvents::MODE_DEFAULT) {
     unsigned event_id = key_to_event(overlay_mode, key_code);
@@ -301,13 +319,8 @@ key_to_event(InputEvents::Mode mode, InputEvents::Mode overlay_mode,
 }
 
 bool
-InputEvents::ProcessKey(Mode mode, unsigned key_code)
+InputEvents::ProcessKey(Mode mode, unsigned key_code) noexcept
 {
-  if (IsAltair() && key_code == 0xF5) {
-    UIActions::SignalShutdown(false);
-    return true;
-  }
-
   if (!global_running)
     return false;
 
@@ -321,6 +334,10 @@ InputEvents::ProcessKey(Mode mode, unsigned key_code)
 #endif
 #endif
 
+  if (Lua::FireKey(key_code)) {
+    //    return true;
+  }
+
   // Which key - can be defined locally or at default (fall back to default)
   unsigned event_id = key_to_event(mode, overlay_mode, key_code);
   if (event_id == 0)
@@ -337,25 +354,25 @@ InputEvents::ProcessKey(Mode mode, unsigned key_code)
   Return = We had a valid key (even if nothing happens because of Bounce)
 */
 bool
-InputEvents::processKey(unsigned key_code)
+InputEvents::processKey(unsigned key_code) noexcept
 {
   return ProcessKey(getModeID(), key_code);
 }
 
 unsigned
-InputEvents::gesture_to_event(const TCHAR *data)
+InputEvents::gesture_to_event(const TCHAR *data) noexcept
 {
   return input_config.Gesture2Event.Get(data, 0);
 }
 
 bool
-InputEvents::IsGesture(const TCHAR *data)
+InputEvents::IsGesture(const TCHAR *data) noexcept
 {
-  return gesture_to_event(data) != 0;
+  return (Lua::IsGesture(data)) || (gesture_to_event(data) != 0);
 }
 
 bool
-InputEvents::processGesture(const TCHAR *data)
+InputEvents::processGesture(const TCHAR *data) noexcept
 {
   // get current mode
   unsigned event_id = gesture_to_event(data);
@@ -364,7 +381,7 @@ InputEvents::processGesture(const TCHAR *data)
     InputEvents::processGo(event_id);
     return true;
   }
-  return false;
+  return Lua::FireGesture(data);
 }
 
 /*
@@ -373,7 +390,7 @@ InputEvents::processGesture(const TCHAR *data)
   Return = TRUE if we have a valid key match
 */
 bool
-InputEvents::processNmea_real(unsigned ne_id)
+InputEvents::processNmea_real(unsigned ne_id) noexcept
 {
   if (!global_running)
     return false;
@@ -389,8 +406,7 @@ InputEvents::processNmea_real(unsigned ne_id)
     InputEvents::processGo(event_id);
     return true;
   }
-
-  return false;
+  return Lua::FireNMEAEvent(ne_id);
 }
 
 /*
@@ -398,7 +414,7 @@ InputEvents::processNmea_real(unsigned ne_id)
   Take virtual inputs from a Glide Computer to do special events
 */
 bool
-InputEvents::processGlideComputer_real(unsigned gce_id)
+InputEvents::processGlideComputer_real(unsigned gce_id) noexcept
 {
   if (!global_running)
     return false;
@@ -416,12 +432,12 @@ InputEvents::processGlideComputer_real(unsigned gce_id)
     return true;
   }
 
-  return false;
+  return Lua::FireGlideComputerEvent(gce_id);
 }
 
 // EXECUTE an Event - lookup event handler and call back - no return
 void
-InputEvents::processGo(unsigned eventid)
+InputEvents::processGo(unsigned eventid) noexcept
 {
   /* eventid 0 is special for "noop" */
 
@@ -437,13 +453,13 @@ InputEvents::processGo(unsigned eventid)
 }
 
 void
-InputEvents::HideMenu()
+InputEvents::HideMenu() noexcept
 {
   setMode(MODE_DEFAULT);
 }
 
 void
-InputEvents::ShowMenu()
+InputEvents::ShowMenu() noexcept
 {
   setMode(MODE_MENU);
   MenuTimeOut = 0;
@@ -451,7 +467,7 @@ InputEvents::ShowMenu()
 }
 
 Menu *
-InputEvents::GetMenu(const TCHAR *mode)
+InputEvents::GetMenu(const TCHAR *mode) noexcept
 {
  int m = input_config.LookupMode(mode);
  if (m >= 0)
@@ -461,7 +477,7 @@ InputEvents::GetMenu(const TCHAR *mode)
 }
 
 void
-InputEvents::ProcessMenuTimer()
+InputEvents::ProcessMenuTimer() noexcept
 {
   if (CommonInterface::main_window->HasDialog())
     /* no menu updates while a dialog is visible */
@@ -477,8 +493,14 @@ InputEvents::ProcessMenuTimer()
 }
 
 void
-InputEvents::ProcessTimer()
+InputEvents::ProcessTimer() noexcept
 {
   DoQueuedEvents();
   ProcessMenuTimer();
 }
+
+void
+InputEvents::eventLockScreen(const TCHAR *mode)
+{
+  ShowLockBox();
+}
