@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@ Copyright_License {
 #include "Writer.hpp"
 
 void
-JSON::WriteString(TextWriter &writer, const char *value)
+JSON::WriteString(BufferedOutputStream &writer, const char *value)
 {
   writer.Write('"');
   // TODO: escape
