@@ -2,7 +2,7 @@
   Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -28,12 +28,12 @@
 #include "Device/Declaration.hpp"
 #include "NMEA/Info.hpp"
 #include "Simulator.hpp"
-#include "OS/FileUtil.hpp"
+#include "system/FileUtil.hpp"
 #include "Formatter/IGCFilenameFormatter.hpp"
 #include "Interface.hpp"
-#include "Util/CharUtil.hpp"
 #include "IGCFileCleanup.hpp"
 #include "IGC/IGCWriter.hpp"
+#include "util/CharUtil.hxx"
 
 #include <tchar.h>
 #include <algorithm>
@@ -77,9 +77,8 @@ LoggerImpl::PreTakeoffBuffer::operator=(const NMEAInfo &src)
 }
 
 LoggerImpl::LoggerImpl()
-  :writer(nullptr)
+  :filename(nullptr), writer(nullptr)
 {
-  filename[0] = 0;
 }
 
 LoggerImpl::~LoggerImpl()
@@ -99,7 +98,9 @@ LoggerImpl::StopLogger(const NMEAInfo &gps_info)
   if (!simulator)
     writer->Sign();
 
-  LogFormat(_T("Logger stopped: %s"), filename);
+  writer->Flush();
+
+  LogFormat(_T("Logger stopped: %s"), filename.c_str());
 
   // Logger off
   delete writer;
@@ -156,7 +157,7 @@ LoggerImpl::LogPoint(const NMEAInfo &gps_info)
 
     // NOTE: clock is only used to set the validity of valid objects to true
     //       for which "1" is sufficient. This kludge needs to be rewritten.
-    tmp_info.clock = fixed(1);
+    tmp_info.clock = 1;
 
     tmp_info.alive.Update(tmp_info.clock);
 
@@ -186,7 +187,7 @@ LoggerImpl::LogPoint(const NMEAInfo &gps_info)
       tmp_info.gps.satellites_used = src.satellites_used;
     }
 
-    tmp_info.gps.hdop = src.hdop;
+    tmp_info.gps.hdop = src.location.IsValid() ? src.hdop : -1;
     tmp_info.gps.real = src.real;
 
     if (src.satellite_ids_available) {
@@ -234,7 +235,7 @@ LoggerImpl::StartLogger(const NMEAInfo &gps_info,
 
   assert(writer == nullptr);
 
-  Directory::Create(LocalPath(filename, _T("logs")));
+  const auto logs_path = MakeLocalPath(_T("logs"));
 
   const BrokenDate today = gps_info.date_time_utc.IsDatePlausible()
     ? (const BrokenDate &)gps_info.date_time_utc
@@ -244,21 +245,21 @@ LoggerImpl::StartLogger(const NMEAInfo &gps_info,
   for (int i = 1; i < 99; i++) {
     FormatIGCFilenameLong(name.buffer(), today, "XCS", logger_id, i);
 
-    LocalPath(filename, _T("logs"), name);
+    filename = AllocatedPath::Build(logs_path, name);
     if (!File::Exists(filename))
       break;  // file not exist, we'll use this name
   }
 
   frecord.Reset();
-  writer = new IGCWriter(filename);
-  if (!writer->IsOpen()) {
-    LogFormat(_T("Failed to create file %s"), filename);
-    delete writer;
-    writer = nullptr;
+
+  try {
+    writer = new IGCWriter(filename);
+  } catch (...) {
+    LogError(std::current_exception());
     return false;
   }
 
-  LogFormat(_T("Logger Started: %s"), filename);
+  LogFormat(_T("Logger Started: %s"), filename.c_str());
   return true;
 }
 
@@ -306,7 +307,7 @@ LoggerImpl::StartLogger(const NMEAInfo &gps_info,
     return;
 
   simulator = gps_info.location_available && !gps_info.gps.real;
-  writer->WriteHeader(gps_info.date_time_utc, decl.pilot_name,
+  writer->WriteHeader(gps_info.date_time_utc, decl.pilot_name, decl.copilot_name,
                       decl.aircraft_type, decl.aircraft_registration,
                       decl.competition_id,
                       logger_id, GetGPSDeviceName(), simulator);
