@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,13 +22,10 @@ Copyright_License {
 */
 
 #include "MainWindow.hpp"
-#include "Startup.hpp"
 #include "MapWindow/GlueMapWindow.hpp"
 #include "PopupMessage.hpp"
 #include "InfoBoxes/InfoBoxManager.hpp"
 #include "InfoBoxes/InfoBoxLayout.hpp"
-#include "Interface.hpp"
-#include "ActionInterface.hpp"
 #include "UIActions.hpp"
 #include "PageActions.hpp"
 #include "Input/InputEvents.hpp"
@@ -42,20 +39,16 @@ Copyright_License {
 #include "Gauge/GaugeFLARM.hpp"
 #include "Gauge/GaugeThermalAssistant.hpp"
 #include "Gauge/GlueGaugeVario.hpp"
-#include "Menu/MenuBar.hpp"
 #include "Form/Form.hpp"
 #include "Widget/Widget.hpp"
 #include "UtilsSystem.hpp"
 #include "Look/GlobalFonts.hpp"
 #include "Look/DefaultFonts.hpp"
 #include "Look/Look.hpp"
-#include "Profile/ProfileKeys.hpp"
-#include "Profile/Profile.hpp"
 #include "ProgressGlue.hpp"
 #include "UIState.hpp"
 #include "DrawThread.hpp"
 #include "UIReceiveBlackboard.hpp"
-#include "Event/Idle.hpp"
 #include "UISettings.hpp"
 #include "Interface.hpp"
 
@@ -63,11 +56,11 @@ Copyright_License {
 #include "Dialogs/Message.hpp"
 #endif
 
-#if !defined(WIN32) && !defined(ANDROID)
+#if !defined(_WIN32) && !defined(ANDROID)
 #include <unistd.h>
 #endif
 
-#if !defined(WIN32) && !defined(ANDROID)
+#if !defined(_WIN32) && !defined(ANDROID)
 #include <unistd.h> /* for execl() */
 #endif
 
@@ -101,13 +94,13 @@ GetBottomWidgetRect(const PixelRect &rc, const Widget *bottom_widget)
     return result;
   }
 
-  const UPixelScalar requested_height = bottom_widget->GetMinimumSize().cy;
-  UPixelScalar height;
+  const unsigned requested_height = bottom_widget->GetMinimumSize().height;
+  unsigned height;
   if (requested_height > 0) {
-    const UPixelScalar max_height = (rc.bottom - rc.top) / 2;
+    const unsigned max_height = rc.GetHeight() / 2;
     height = std::min(max_height, requested_height);
   } else {
-    const UPixelScalar recommended_height = (rc.bottom - rc.top) / 3;
+    const unsigned recommended_height = rc.GetHeight() / 3;
     height = recommended_height;
   }
 
@@ -127,26 +120,6 @@ GetMapRectAbove(const PixelRect &rc, const PixelRect &bottom_rect)
   return result;
 }
 
-MainWindow::MainWindow()
-  :look(nullptr),
-#ifdef HAVE_SHOW_MENU_BUTTON
-   show_menu_button(nullptr),
-#endif
-   map(nullptr), bottom_widget(nullptr), widget(nullptr), vario(*this),
-   traffic_gauge(*this),
-   suppress_traffic_gauge(false), force_traffic_gauge(false),
-   thermal_assistant(*this),
-   dragging(false),
-   popup(nullptr),
-   timer(*this),
-   FullScreen(false),
-#ifndef ENABLE_OPENGL
-   draw_suspended(false),
-#endif
-   restore_page_pending(false)
-{
-}
-
 /**
  * Destructor of the MainWindow-Class
  * @return
@@ -157,13 +130,12 @@ MainWindow::~MainWindow()
 }
 
 void
-MainWindow::Create(PixelSize size, TopWindowStyle style)
+MainWindow::Create(PixelSize size, UI::TopWindowStyle style)
 {
-  style.EnableDoubleClicks();
   SingleWindow::Create(title, size, style);
 }
 
-gcc_noreturn
+[[noreturn]]
 static void
 FatalError(const TCHAR *msg)
 {
@@ -177,7 +149,7 @@ FatalError(const TCHAR *msg)
   LogFormat(_T("%s"), msg);
 
   /* now try to get a GUI error message out to the user */
-#ifdef WIN32
+#ifdef _WIN32
   MessageBox(nullptr, msg, _T("XCSoar"), MB_ICONEXCLAMATION|MB_OK);
 #elif !defined(ANDROID) && !defined(KOBO)
   execl("/usr/bin/xmessage", "xmessage", msg, nullptr);
@@ -186,7 +158,7 @@ FatalError(const TCHAR *msg)
   exit(EXIT_FAILURE);
 }
 
-gcc_noreturn
+[[noreturn]]
 static void
 NoFontsAvailable()
 {
@@ -197,7 +169,12 @@ void
 MainWindow::Initialise()
 {
   Layout::Initialize(GetSize(),
-                     CommonInterface::GetUISettings().GetPercentScale());
+                     CommonInterface::GetUISettings().GetPercentScale(),
+                     CommonInterface::GetUISettings().custom_dpi);
+#ifdef DRAW_MOUSE_CURSOR
+  SetCursorSize(CommonInterface::GetDisplaySettings().cursor_size);
+  SetCursorColorsInverted(CommonInterface::GetDisplaySettings().invert_cursor_colors);
+#endif
 
   LogFormat("Initialise fonts");
   if (!Fonts::Initialize()) {
@@ -216,11 +193,9 @@ MainWindow::InitialiseConfigured()
 {
   const UISettings &ui_settings = CommonInterface::GetUISettings();
 
-#ifndef GNAV
   if (ui_settings.scale != 100)
     /* call Initialise() again to reload fonts with the new scale */
     Initialise();
-#endif
 
   PixelRect rc = GetClientRect();
 
@@ -230,9 +205,9 @@ MainWindow::InitialiseConfigured()
   assert(look != nullptr);
   look->InitialiseConfigured(CommonInterface::GetUISettings(),
                              Fonts::map, Fonts::map_bold,
-                             ib_layout.control_size.cx);
+                             ib_layout.control_size.width);
 
-  InfoBoxManager::Create(*this, ib_layout, look->info_box, look->units);
+  InfoBoxManager::Create(*this, ib_layout, look->info_box);
   map_rect = ib_layout.remaining;
 
   ButtonLabel::CreateButtonLabels(*this, look->dialog.button);
@@ -304,7 +279,7 @@ MainWindow::ReinitialiseLayout_vario(const InfoBoxLayout::Layout &layout)
 
   if (!vario.IsDefined())
     vario.Set(new GlueGaugeVario(CommonInterface::GetLiveBlackboard(),
-                                 look->vario, look->units));
+                                 look->vario));
 
   vario.Move(layout.vario);
   vario.Show();
@@ -316,9 +291,29 @@ void
 MainWindow::ReinitialiseLayoutTA(PixelRect rc,
                                  const InfoBoxLayout::Layout &layout)
 {
-  UPixelScalar sz = std::min(layout.control_size.cy,
-                             layout.control_size.cx) * 2;
-  rc.right = rc.left + sz;
+  unsigned sz = std::min(layout.control_size.height,
+                         layout.control_size.width) * 2;
+
+  switch (CommonInterface::GetUISettings().thermal_assistant_position) {
+  case (UISettings::ThermalAssistantPosition::BOTTOM_LEFT_AVOID_IB):
+    rc.bottom = GetMainRect().bottom;
+    rc.left = GetMainRect().left;
+    rc.right = rc.left + sz;
+    break;
+  case (UISettings::ThermalAssistantPosition::BOTTOM_RIGHT_AVOID_IB):
+    rc.bottom = GetMainRect().bottom;
+    rc.right = GetMainRect().right;
+    rc.left = rc.right - sz;
+    break;
+  case (UISettings::ThermalAssistantPosition::BOTTOM_RIGHT):
+    rc.right = GetMainRect().right;
+    rc.left = rc.right - sz;
+    break;
+  default: // BOTTOM_LEFT
+    rc.left = GetMainRect().left;
+    rc.right = rc.left + sz;
+    break;
+  } 
   rc.top = rc.bottom - sz;
   thermal_assistant.Move(rc);
 }
@@ -347,9 +342,9 @@ MainWindow::ReinitialiseLayout()
   const InfoBoxLayout::Layout ib_layout =
     InfoBoxLayout::Calculate(rc, ui_settings.info_boxes.geometry);
 
-  look->ReinitialiseLayout(ib_layout.control_size.cx);
+  look->ReinitialiseLayout(ib_layout.control_size.width);
 
-  InfoBoxManager::Create(*this, ib_layout, look->info_box, look->units);
+  InfoBoxManager::Create(*this, ib_layout, look->info_box);
   InfoBoxManager::ProcessTimer();
   map_rect = ib_layout.remaining;
 
@@ -391,7 +386,8 @@ MainWindow::ReinitialiseLayout()
 }
 
 void 
-MainWindow::ReinitialiseLayout_flarm(PixelRect rc, const InfoBoxLayout::Layout ib_layout)
+MainWindow::ReinitialiseLayout_flarm(PixelRect rc,
+                                     const InfoBoxLayout::Layout &ib_layout)
 {
   TrafficSettings::GaugeLocation val =
     CommonInterface::GetUISettings().traffic.gauge_location;
@@ -415,40 +411,40 @@ MainWindow::ReinitialiseLayout_flarm(PixelRect rc, const InfoBoxLayout::Layout i
 
   switch (val) {
   case TrafficSettings::GaugeLocation::TopLeft:
-    rc.right = rc.left + ib_layout.control_size.cx * 2;
+    rc.right = rc.left + ib_layout.control_size.width * 2;
     ++rc.left;
-    rc.bottom = rc.top + ib_layout.control_size.cy * 2;
+    rc.bottom = rc.top + ib_layout.control_size.height * 2;
     ++rc.top;
     break;
 
   case TrafficSettings::GaugeLocation::TopRight:
-    rc.left = rc.right - ib_layout.control_size.cx * 2 + 1;
-    rc.bottom = rc.top + ib_layout.control_size.cy * 2;
+    rc.left = rc.right - ib_layout.control_size.width * 2 + 1;
+    rc.bottom = rc.top + ib_layout.control_size.height * 2;
     ++rc.top;
     break;
 
   case TrafficSettings::GaugeLocation::BottomLeft:
-    rc.right = rc.left + ib_layout.control_size.cx * 2;
+    rc.right = rc.left + ib_layout.control_size.width * 2;
     ++rc.left;
-    rc.top = rc.bottom - ib_layout.control_size.cy * 2 + 1;
+    rc.top = rc.bottom - ib_layout.control_size.height * 2 + 1;
     break;
 
   case TrafficSettings::GaugeLocation::CentreTop:
-    rc.left = (rc.left + rc.right) / 2 - ib_layout.control_size.cx;
-    rc.right = rc.left + ib_layout.control_size.cx * 2 - 1;
-    rc.bottom = rc.top + ib_layout.control_size.cy * 2;
+    rc.left = (rc.left + rc.right) / 2 - ib_layout.control_size.width;
+    rc.right = rc.left + ib_layout.control_size.width * 2 - 1;
+    rc.bottom = rc.top + ib_layout.control_size.height * 2;
     ++rc.top;
     break;
 
   case TrafficSettings::GaugeLocation::CentreBottom:
-    rc.left = (rc.left + rc.right) / 2 - ib_layout.control_size.cx;
-    rc.right = rc.left + ib_layout.control_size.cx * 2 - 1;
-    rc.top = rc.bottom - ib_layout.control_size.cy * 2 + 1;
+    rc.left = (rc.left + rc.right) / 2 - ib_layout.control_size.width;
+    rc.right = rc.left + ib_layout.control_size.width * 2 - 1;
+    rc.top = rc.bottom - ib_layout.control_size.height * 2 + 1;
     break;
 
   default:    // aka flBottomRight
-    rc.left = rc.right - ib_layout.control_size.cx * 2 + 1;
-    rc.top = rc.bottom - ib_layout.control_size.cy * 2 + 1;
+    rc.left = rc.right - ib_layout.control_size.width * 2 + 1;
+    rc.top = rc.bottom - ib_layout.control_size.height * 2 + 1;
     break;
   }
 
@@ -466,7 +462,7 @@ MainWindow::Destroy()
 void
 MainWindow::FinishStartup()
 {
-  timer.Schedule(500); // 2 times per second
+  timer.Schedule(std::chrono::milliseconds(500)); // 2 times per second
 
   ResumeThreads();
 }
@@ -522,7 +518,8 @@ void
 MainWindow::OnResize(PixelSize new_size)
 {
   Layout::Initialize(new_size,
-                     CommonInterface::GetUISettings().GetPercentScale());
+                     CommonInterface::GetUISettings().GetPercentScale(),
+                     CommonInterface::GetUISettings().custom_dpi);
 
   SingleWindow::OnResize(new_size);
 
@@ -533,20 +530,6 @@ MainWindow::OnResize(PixelSize new_size)
   ProgressGlue::Move(rc);
 }
 
-#ifdef USE_GDI
-
-bool
-MainWindow::OnActivate()
-{
-  SingleWindow::OnActivate();
-
-  Fullscreen();
-
-  return true;
-}
-
-#endif
-
 void
 MainWindow::OnSetFocus()
 {
@@ -583,24 +566,24 @@ MainWindow::OnCancelMode()
 }
 
 bool
-MainWindow::OnMouseDown(PixelScalar x, PixelScalar y)
+MainWindow::OnMouseDown(PixelPoint p)
 {
-  if (SingleWindow::OnMouseDown(x, y))
+  if (SingleWindow::OnMouseDown(p))
     return true;
 
   if (!dragging && !HasDialog()) {
     dragging = true;
     SetCapture();
-    gestures.Start(x, y, Layout::Scale(20));
+    gestures.Start(p, Layout::Scale(20));
   }
 
   return true;
 }
 
 bool
-MainWindow::OnMouseUp(PixelScalar x, PixelScalar y)
+MainWindow::OnMouseUp(PixelPoint p)
 {
-  if (SingleWindow::OnMouseUp(x, y))
+  if (SingleWindow::OnMouseUp(p))
     return true;
 
   if (dragging) {
@@ -615,9 +598,9 @@ MainWindow::OnMouseUp(PixelScalar x, PixelScalar y)
 }
 
 bool
-MainWindow::OnMouseDouble(PixelScalar x, PixelScalar y)
+MainWindow::OnMouseDouble(PixelPoint p)
 {
-  if (SingleWindow::OnMouseDouble(x, y))
+  if (SingleWindow::OnMouseDouble(p))
     return true;
 
   StopDragging();
@@ -628,13 +611,13 @@ MainWindow::OnMouseDouble(PixelScalar x, PixelScalar y)
 }
 
 bool
-MainWindow::OnMouseMove(PixelScalar x, PixelScalar y, unsigned keys)
+MainWindow::OnMouseMove(PixelPoint p, unsigned keys)
 {
-  if (SingleWindow::OnMouseMove(x, y, keys))
+  if (SingleWindow::OnMouseMove(p, keys))
     return true;
 
   if (dragging)
-    gestures.Update(x, y);
+    gestures.Update(p);
 
   return true;
 }
@@ -648,17 +631,14 @@ MainWindow::OnKeyDown(unsigned key_code)
     SingleWindow::OnKeyDown(key_code);
 }
 
-bool
-MainWindow::OnTimer(WindowTimer &_timer)
+void
+MainWindow::RunTimer() noexcept
 {
-  if (_timer != timer)
-    return SingleWindow::OnTimer(_timer);
-
   ProcessTimer();
 
   UpdateGaugeVisibility();
 
-  if (!CommonInterface::GetUISettings().enable_thermal_assistant_gauge) {
+  if (CommonInterface::GetUISettings().thermal_assistant_position == UISettings::ThermalAssistantPosition::OFF) {
     thermal_assistant.Clear();
   } else if (!CommonInterface::Calculated().circling ||
              InputEvents::IsFlavour(_T("TA"))) {
@@ -678,29 +658,25 @@ MainWindow::OnTimer(WindowTimer &_timer)
   }
 
   battery_timer.Process();
-
-  return true;
 }
 
-bool
-MainWindow::OnUser(unsigned id)
+void
+MainWindow::OnGpsNotify() noexcept
 {
-  switch ((Command)id) {
-  case Command::GPS_UPDATE:
-    UIReceiveSensorData();
-    return true;
+  UIReceiveSensorData();
+}
 
-  case Command::CALCULATED_UPDATE:
-    UIReceiveCalculatedData();
-    return true;
+void
+MainWindow::OnCalculatedNotify() noexcept
+{
+  UIReceiveCalculatedData();
+}
 
-  case Command::RESTORE_PAGE:
-    if (restore_page_pending)
-      PageActions::Restore();
-    return true;
-  }
-
-  return false;
+void
+MainWindow::OnRestorePageNotify() noexcept
+{
+  if (restore_page_pending)
+    PageActions::Restore();
 }
 
 void
@@ -714,7 +690,9 @@ MainWindow::OnDestroy()
   SingleWindow::OnDestroy();
 }
 
-bool MainWindow::OnClose() {
+bool
+MainWindow::OnClose() noexcept
+{
   if (HasDialog() || !IsRunning())
     /* no shutdown dialog if XCSoar hasn't completed initialization
        yet (e.g. if we are in the simulator prompt) */
@@ -844,7 +822,7 @@ MainWindow::DeferredRestorePage()
     return;
 
   restore_page_pending = true;
-  SendUser((unsigned)Command::RESTORE_PAGE);
+  restore_page_notify.SendNotification();
 }
 
 void
@@ -1039,7 +1017,7 @@ MainWindow::ToggleForceFLARMRadar()
 #ifdef ANDROID
 
 void
-MainWindow::OnPause()
+MainWindow::OnPause() noexcept
 {
   if (!IsRunning() && HasDialog())
     /* suspending before initialization has finished doesn't leave
