@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,19 +24,14 @@ Copyright_License {
 #include "ProcessTimer.hpp"
 #include "Interface.hpp"
 #include "ActionInterface.hpp"
-#include "Protection.hpp"
 #include "Input/InputQueue.hpp"
 #include "Input/InputEvents.hpp"
-#include "Device/device.hpp"
 #include "Device/MultipleDevices.hpp"
-#include "Screen/Blank.hpp"
-#include "UtilsSystem.hpp"
 #include "Blackboard/DeviceBlackboard.hpp"
 #include "Components.hpp"
-#include "Time/PeriodClock.hpp"
+#include "time/PeriodClock.hpp"
 #include "MainWindow.hpp"
 #include "PopupMessage.hpp"
-#include "Asset.hpp"
 #include "Simulator.hpp"
 #include "Replay/Replay.hpp"
 #include "InfoBoxes/InfoBoxManager.hpp"
@@ -44,26 +39,11 @@ Copyright_License {
 #include "BallastDumpManager.hpp"
 #include "Operation/Operation.hpp"
 #include "Tracking/TrackingGlue.hpp"
-#include "Operation/MessageOperationEnvironment.hpp"
-#include "Event/Idle.hpp"
+#include "ui/event/Idle.hpp"
 #include "Dialogs/Tracking/CloudEnableDialog.hpp"
 
-#ifdef _WIN32_WCE
 static void
-HeapCompact()
-{
-  static int iheapcompact = 0;
-  // called 2 times per second, compact heap every minute.
-  iheapcompact++;
-  if (iheapcompact == 120) {
-    MyCompactHeaps();
-    iheapcompact = 0;
-  }
-}
-#endif
-
-static void
-MessageProcessTimer()
+MessageProcessTimer() noexcept
 {
   // don't display messages if airspace warning dialog is active
   if (CommonInterface::main_window->popup != nullptr &&
@@ -77,13 +57,12 @@ MessageProcessTimer()
  * defined in settings
  */
 static void
-SystemClockTimer()
+SystemClockTimer() noexcept
 {
-#ifdef WIN32
+#ifdef _WIN32
   const NMEAInfo &basic = CommonInterface::Basic();
 
-  // Altair doesn't have a battery-backed up realtime clock,
-  // so as soon as we get a fix for the first time, set the
+  // as soon as we get a fix for the first time, set the
   // system clock to the GPS time.
   static bool sysTimeInitialised = false;
 
@@ -106,18 +85,6 @@ SystemClockTimer()
     sysTime.wMilliseconds = 0;
     ::SetSystemTime(&sysTime);
 
-#if defined(_WIN32_WCE) && defined(GNAV)
-    TIME_ZONE_INFORMATION tzi;
-    tzi.Bias = - CommonInterface::GetComputerSettings().utc_offset.AsMinutes();
-    _tcscpy(tzi.StandardName,TEXT("Altair"));
-    tzi.StandardDate.wMonth= 0; // disable daylight savings
-    tzi.StandardBias = 0;
-    _tcscpy(tzi.DaylightName,TEXT("Altair"));
-    tzi.DaylightDate.wMonth= 0; // disable daylight savings
-    tzi.DaylightBias = 0;
-
-    SetTimeZoneInformation(&tzi);
-#endif
     sysTimeInitialised =true;
   } else if (!basic.alive)
     /* set system clock again after a device reconnect; the new device
@@ -129,26 +96,20 @@ SystemClockTimer()
 }
 
 static void
-SystemProcessTimer()
+SystemProcessTimer() noexcept
 {
-#ifdef _WIN32_WCE
-  HeapCompact();
-#endif
-
   SystemClockTimer();
-
-  CheckDisplayTimeOut(false);
 }
 
 static void
-BlackboardProcessTimer()
+BlackboardProcessTimer() noexcept
 {
   device_blackboard->ExpireWallClock();
   XCSoarInterface::ExchangeBlackboard();
 }
 
 static void
-BallastDumpProcessTimer()
+BallastDumpProcessTimer() noexcept
 {
   ComputerSettings &settings_computer =
     CommonInterface::SetComputerSettings();
@@ -173,35 +134,35 @@ BallastDumpProcessTimer()
 }
 
 static void
-ProcessAutoBugs()
+ProcessAutoBugs() noexcept
 {
   /**
    * Increase the bugs value every hour.
    */
-  static constexpr fixed interval(3600);
+  static constexpr double interval(3600);
 
   /**
    * Decrement the bugs setting by 1%.
    */
-  static constexpr fixed decrement(0.01);
+  static constexpr double decrement(0.01);
 
   /**
    * Don't go below this bugs setting.
    */
-  static constexpr fixed min_bugs(0.7);
+  static constexpr double min_bugs(0.7);
 
   /**
    * The time stamp (from FlyingState::flight_time) when we last
    * increased the bugs value automatically.
    */
-  static fixed last_auto_bugs;
+  static double last_auto_bugs;
 
   const FlyingState &flight = CommonInterface::Calculated().flight;
   const PolarSettings &polar = CommonInterface::GetComputerSettings().polar;
 
   if (!flight.flying)
     /* reset when not flying */
-    last_auto_bugs = fixed(0);
+    last_auto_bugs = 0;
   else if (!polar.auto_bugs)
     /* feature is disabled */
     last_auto_bugs = flight.flight_time;
@@ -213,7 +174,7 @@ ProcessAutoBugs()
 }
 
 static void
-SettingsProcessTimer()
+SettingsProcessTimer() noexcept
 {
   CloudEnableDialog();
   BallastDumpProcessTimer();
@@ -221,7 +182,7 @@ SettingsProcessTimer()
 }
 
 static void
-CommonProcessTimer()
+CommonProcessTimer() noexcept
 {
   BlackboardProcessTimer();
 
@@ -235,7 +196,7 @@ CommonProcessTimer()
 }
 
 static void
-ConnectionProcessTimer()
+ConnectionProcessTimer() noexcept
 {
   if (devices == nullptr)
     return;
@@ -273,7 +234,7 @@ ConnectionProcessTimer()
 }
 
 void
-ProcessTimer()
+ProcessTimer() noexcept
 {
   CommonProcessTimer();
 
@@ -294,7 +255,7 @@ ProcessTimer()
 
     if (replay && replay->IsActive()) {
       m_clock.Update();
-    } else if (m_clock.Elapsed() >= 1000) {
+    } else if (m_clock.Elapsed() >= std::chrono::seconds(1)) {
       m_clock.Update();
       device_blackboard->ProcessSimulation();
     } else if (!m_clock.IsDefined())
