@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -26,6 +26,7 @@ Copyright_License {
 namespace ProfileKeys {
 
 const char UIScale[] = "UIScale";
+const char CustomDPI[] = "CustomDPI";
 const char Password[] = "Password";
 const char AirspaceWarning[] = "AirspaceWarn";
 const char AirspaceWarningDialog[] = "AirspaceWarnDialog";
@@ -86,6 +87,7 @@ const char AcknowledgementTime[] = "AcknowledgementTime";
 const char AirfieldFile[] = "AirfieldFile"; // pL
 const char AirspaceFile[] = "AirspaceFile"; // pL
 const char AdditionalAirspaceFile[] = "AdditionalAirspaceFile"; // pL
+const char FlarmFile[] = "FlarmFile";
 const char PolarFile[] = "PolarFile"; // pL
 const char WaypointFile[] = "WPFile"; // pL
 const char AdditionalWaypointFile[] = "AdditionalWPFile"; // pL
@@ -93,6 +95,7 @@ const char WatchedWaypointFile[] = "WatchedWPFile"; // pL
 const char LanguageFile[] = "LanguageFile"; // pL
 const char InputFile[] = "InputFile"; // pL
 const char PilotName[] = "PilotName";
+const char CoPilotName[] = "CoPilotName";
 const char AircraftType[] = "AircraftType";
 const char AircraftReg[] = "AircraftReg";
 const char CompetitionId[] = "AircraftRego";
@@ -106,7 +109,6 @@ const char SoundModes[] = "SoundModes";
 const char NettoSpeed[] = "NettoSpeed";
 const char AccelerometerZero[] = "AccelerometerZero";
 
-const char AutoBlank[] = "AutoBlank";
 const char AverEffTime[] = "AverEffTime";
 const char VarioGauge[] = "VarioGauge";
 
@@ -130,10 +132,11 @@ const char AppInfoBoxColors[] = "AppInfoBoxColors";
 const char TeamcodeRefWaypoint[] = "TeamcodeRefWaypoint";
 const char AppInfoBoxBorder[] = "AppInfoBoxBorder";
 const char ShowMenuButton[] = "ShowMenuButton";
-
-const char AppInfoBoxModel[] = "AppInfoBoxModel"; // VENTA-ADDON MODEL CONFIG
+const char CursorSize[] = "CursorSize";
+const char CursorColorsInverted[] = "CursorColorsInverted";
 
 const char AppAveNeedle[] = "AppAveNeedle";
+const char AppAveThermalNeedle[] = "AppAveThermalNeedle";
 
 const char AutoAdvance[] = "AutoAdvance";
 const char UTCOffset[] = "UTCOffset";
@@ -148,19 +151,11 @@ const char EnableFLARMMap[] = "EnableFLARMDisplay";
 const char EnableFLARMGauge[] = "EnableFLARMGauge";
 const char AutoCloseFlarmDialog[] = "AutoCloseFlarmDialog";
 const char EnableTAGauge[] = "EnableTAGauge";
+const char TAPosition[] = "TAPosition";
 const char EnableThermalProfile[] = "EnableThermalProfile";
 const char GliderScreenPosition[] = "GliderScreenPosition";
 const char SetSystemTimeFromGPS[] = "SetSystemTimeFromGPS";
 
-const char VoiceClimbRate[] = "VoiceClimbRate";
-const char VoiceTerrain[] = "VoiceTerrain";
-const char VoiceWaypointDistance[] = "VoiceWaypointDistance";
-const char VoiceTaskAltitudeDifference[] = "VoiceTaskAltitudeDifference";
-const char VoiceMacCready[] = "VoiceMacCready";
-const char VoiceNewWaypoint[] = "VoiceNewWaypoint";
-const char VoiceInSector[] = "VoiceInSector";
-const char VoiceAirspace[] = "VoiceAirspace";
-
 const char FinishMinHeight[] = "FinishMinHeight";
 const char FinishHeightRef[] = "FinishHeightRef";
 const char StartMaxHeight[] = "StartMaxHeight";
@@ -188,7 +183,9 @@ const char AbortTaskMode[] = "AbortTaskMode";
 const char AutoMcMode[] = "AutoMcMode";
 const char AutoMc[] = "AutoMc";
 const char EnableExternalTriggerCruise[] = "EnableExternalTriggerCruise";
-const char OLCRules[] = "OLCRules";
+const char CruiseToCirclingModeSwitchThreshold[] = "CruiseToCirclingModeSwitchThreshold";
+const char CirclingToCruiseModeSwitchThreshold[] = "CirclingToCruiseModeSwitchThreshold";
+const char OLCRules[] = "OLCRules"; // legacy name, key contains contest rules
 const char PredictContest[] = "PredictContest";
 const char Handicap[] = "Handicap";
 const char SnailWidthScale[] = "SnailWidthScale";
@@ -245,23 +242,30 @@ const char AircraftSymbol[] = "AircraftSymbol";
 
 const char FlarmLocation[] = "FlarmLocation";
 
-const char TrackingInterval[] = "TrackingInterval";
-const char TrackingVehicleType[] = "TrackingVehicleType";
-const char TrackingVehicleName[] = "TrackingVehicleName";
 const char SkyLinesTrackingEnabled[] = "SkyLinesTrackingEnabled";
 const char SkyLinesRoaming[] = "SkyLinesRoaming";
 const char SkyLinesTrackingInterval[] = "SkyLinesTrackingInterval";
 const char SkyLinesTrafficEnabled[] = "SkyLinesTrafficEnabled";
 const char SkyLinesNearTrafficEnabled[] = "SkyLinesNearTrafficEnabled";
+const char SkyLinesTrafficMapMode[] = "SkyLinesTrafficMapMode";
 const char SkyLinesTrackingKey[] = "SkyLinesTrackingKey";
 
 const char CloudEnabled[] = "CloudEnabled";
+const char CloudShowThermals[] = "CloudShowThermals";
 const char CloudKey[] = "CloudKey";
 
 const char LiveTrack24Enabled[] = "LiveTrack24Enabled";
 const char LiveTrack24Server[] = "LiveTrack24Server";
 const char LiveTrack24Username[] = "LiveTrack24Username";
 const char LiveTrack24Password[] = "LiveTrack24Password";
+const char LiveTrack24TrackingInterval[] = "TrackingInterval";
+const char LiveTrack24TrackingVehicleType[] = "TrackingVehicleType";
+const char LiveTrack24TrackingVehicleName[] = "TrackingVehicleName";
+
+const char PCMetUsername[] = "PCMetUsername";
+const char PCMetPassword[] = "PCMetPassword";
+const char PCMetFtpUsername[] = "PCMetFtpUsername";
+const char PCMetFtpPassword[] = "PCMetFtpPassword";
 
 const char EnableLocationMapItem[] = "EnableLocationMapItem";
 const char EnableArrivalAltitudeMapItem[] = "EnableArrivalAltitudeMapItem";
@@ -279,4 +283,6 @@ const char PagesDistinctZoom[] = "PagesDistinctZoom";
 
 const char WaveAssistant[] = "WaveAssistant";
 
+const char MasterAudioVolume[] = "MasterAudioVolume";
+
 }
