@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,11 +24,11 @@ Copyright_License {
 #ifndef XCSOAR_PROFILE_MAP_HPP
 #define XCSOAR_PROFILE_MAP_HPP
 
-#include "Util/StringBuffer.hxx"
-#include "Math/fixed.hpp"
+#include "util/StringBuffer.hxx"
+#include "util/Compiler.h"
 
 #include <tchar.h>
-#include <stdint.h>
+#include <cstdint>
 
 class KeyValueFileWriter;
 
@@ -54,7 +54,7 @@ namespace Profile {
    * profile), or default_value if the key does not exist
    */
   gcc_pure
-  const char *Get(const char *key, const char *default_value=NULL);
+  const char *Get(const char *key, const char *default_value=nullptr);
 
   /**
    * Reads a value from the profile map
@@ -62,7 +62,7 @@ namespace Profile {
    * @param value Pointer to the output buffer
    * @param max_size Maximum size of the output buffer
    */
-  bool Get(const char *key, TCHAR *value, size_t max_size);
+  bool Get(const char *key, TCHAR *value, std::size_t max_size);
 
   /**
    * Writes a value to the profile map
@@ -81,7 +81,7 @@ namespace Profile {
   bool Get(const char *key, unsigned &value);
   bool Get(const char *key, uint16_t &value);
   bool Get(const char *key, uint8_t &value);
-  bool Get(const char *key, fixed &value);
+  bool Get(const char *key, double &value);
 
   template<typename T>
   static inline bool GetEnum(const char *key, T &value)
@@ -102,7 +102,7 @@ namespace Profile {
   void Set(const char *key, int value);
   void Set(const char *key, long value);
   void Set(const char *key, unsigned value);
-  void Set(const char *key, fixed value);
+  void Set(const char *key, double value);
 
   template<typename T>
   static inline void SetEnum(const char *key, T value)
@@ -110,9 +110,9 @@ namespace Profile {
     Set(key, (int)value);
   }
 
-  template<size_t max>
+  template<std::size_t max>
   static inline bool
-  Get(const char *key, StringBuffer<TCHAR, max> &value)
+  Get(const char *key, BasicStringBuffer<TCHAR, max> &value)
   {
     return Get(key, value.data(), value.capacity());
   }
