@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,11 +22,13 @@ Copyright_License {
 */
 
 #include "Map.hpp"
-#include "Util/UTF8.hpp"
-#include "Util/TruncateString.hpp"
-#include "Util/Macros.hpp"
+#include "util/UTF8.hpp"
+#include "util/TruncateString.hpp"
+#include "util/Macros.hpp"
 
 #ifdef _UNICODE
+#include "util/Macros.hpp"
+
 #include <windows.h>
 #endif
 
