@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -23,12 +23,11 @@ Copyright_License {
 
 #include "FlightStatisticsRenderer.hpp"
 #include "ChartRenderer.hpp"
-#include "Util/Macros.hpp"
-#include "Util/StringUtil.hpp"
 #include "Look/MapLook.hpp"
 #include "Task/ProtectedTaskManager.hpp"
+#include "Engine/Task/TaskManager.hpp"
 #include "Engine/Task/Ordered/OrderedTask.hpp"
-#include "Screen/Canvas.hpp"
+#include "ui/canvas/Canvas.hpp"
 #include "Screen/Layout.hpp"
 #include "NMEA/Info.hpp"
 #include "NMEA/Derived.hpp"
@@ -43,13 +42,12 @@ Copyright_License {
 #include "Renderer/TaskPointRenderer.hpp"
 #include "Renderer/OZRenderer.hpp"
 #include "Renderer/AircraftRenderer.hpp"
+#include "Renderer/MapScaleRenderer.hpp"
 #include "Engine/Contest/Solvers/Retrospective.hpp"
 #include "Computer/Settings.hpp"
 
 #include <algorithm>
 
-#include <stdio.h>
-
 using std::max;
 
 FlightStatisticsRenderer::FlightStatisticsRenderer(const ChartLook &_chart_look,
@@ -87,20 +85,23 @@ FlightStatisticsRenderer::DrawContestTriangle(Canvas &canvas, const Projection &
 }
 
 void
-FlightStatisticsRenderer::RenderOLC(Canvas &canvas, const PixelRect rc,
-                                    const NMEAInfo &nmea_info,
-                                    const ComputerSettings &settings_computer,
-                                    const MapSettings &settings_map,
-                                    const ContestStatistics &contest,
-                                    const TraceComputer &trace_computer,
-                                    const Retrospective &retrospective) const
+FlightStatisticsRenderer::RenderContest(Canvas &canvas, const PixelRect rc,
+                                        const NMEAInfo &nmea_info,
+                                        const ComputerSettings &settings_computer,
+                                        const MapSettings &settings_map,
+                                        const ContestStatistics &contest,
+                                        const TraceComputer &trace_computer,
+                                        const Retrospective &retrospective) const
 {
+  ChartRenderer chart(chart_look, canvas, rc);
+  chart.Begin();
+
   if (!trail_renderer.LoadTrace(trace_computer)) {
-    ChartRenderer chart(chart_look, canvas, rc);
     chart.DrawNoData();
     return;
   }
 
+  const PixelRect &rc_chart = chart.GetChartRect();
   GeoBounds bounds(nmea_info.location);
   trail_renderer.ScanBounds(bounds);
 
@@ -113,7 +114,7 @@ FlightStatisticsRenderer::RenderOLC(Canvas &canvas, const PixelRect rc,
     }
   }
 
-  const ChartProjection proj(rc, TaskProjection(bounds));
+  const ChartProjection proj(rc_chart, TaskProjection(bounds));
 
   {
     // draw place names found in the retrospective task
@@ -122,14 +123,12 @@ FlightStatisticsRenderer::RenderOLC(Canvas &canvas, const PixelRect rc,
     canvas.SetBackgroundTransparent();
 
     for (const auto &i : retrospective.getNearWaypointList()) {
-      RasterPoint wp_pos = proj.GeoToScreen(i.waypoint.location);
-      canvas.DrawText(wp_pos.x,
-                      wp_pos.y,
-                      i.waypoint.name.c_str());
+      auto wp_pos = proj.GeoToScreen(i.waypoint->location);
+      canvas.DrawText(wp_pos, i.waypoint->name.c_str());
     }
   }
 
-  RasterPoint aircraft_pos = proj.GeoToScreen(nmea_info.location);
+  auto aircraft_pos = proj.GeoToScreen(nmea_info.location);
   AircraftRenderer::Draw(canvas, settings_map, map_look.aircraft,
                          nmea_info.attitude.heading, aircraft_pos);
 
@@ -163,13 +162,29 @@ FlightStatisticsRenderer::RenderOLC(Canvas &canvas, const PixelRect rc,
     DrawContestSolution(canvas, proj, contest, 0);
     DrawContestTriangle(canvas, proj, contest, 1);
     break;
+
+  case Contest::WEGLIDE_FREE:
+    DrawContestSolution(canvas, proj, contest, 0);
+    break;
+
+  case Contest::WEGLIDE_DISTANCE:
+  case Contest::WEGLIDE_FAI:
+
+  case Contest::WEGLIDE_OR:
+    DrawContestSolution(canvas, proj, contest, 0);
+    break;
+
   }
+
+  RenderMapScale(canvas, proj, rc_chart, map_look.overlay);
+
+  chart.Finish();
 }
 
 void
-FlightStatisticsRenderer::CaptionOLC(TCHAR *sTmp,
-                                     const ContestSettings &settings,
-                                     const DerivedInfo &derived)
+FlightStatisticsRenderer::CaptionContest(TCHAR *sTmp,
+                                         const ContestSettings &settings,
+                                         const DerivedInfo &derived)
 {
   if (settings.contest == Contest::OLC_PLUS) {
     const ContestResult& result =
@@ -250,19 +265,22 @@ FlightStatisticsRenderer::RenderTask(Canvas &canvas, const PixelRect rc,
                                      const TraceComputer *trace_computer) const
 {
   ChartRenderer chart(chart_look, canvas, rc);
+  chart.Begin();
 
   ChartProjection proj;
 
+  const PixelRect &rc_chart = chart.GetChartRect();
+
   {
     ProtectedTaskManager::Lease task_manager(_task_manager);
     const OrderedTask &task = task_manager->GetOrderedTask();
 
-    if (!task.CheckTask()) {
+    if (IsError(task.CheckTask())) {
       chart.DrawNoData();
       return;
     }
 
-    proj.Set(rc, task);
+    proj.Set(rc_chart, task);
 
     OZRenderer ozv(map_look.task, map_look.airspace, settings_map.airspace);
     TaskPointRenderer tpv(canvas, proj, map_look.task,
@@ -278,10 +296,14 @@ FlightStatisticsRenderer::RenderTask(Canvas &canvas, const PixelRect rc,
     trail_renderer.Draw(canvas, *trace_computer, proj, 0);
 
   if (nmea_info.location_available) {
-    RasterPoint aircraft_pos = proj.GeoToScreen(nmea_info.location);
+    auto aircraft_pos = proj.GeoToScreen(nmea_info.location);
     AircraftRenderer::Draw(canvas, settings_map, map_look.aircraft,
                            nmea_info.attitude.heading, aircraft_pos);
   }
+
+  RenderMapScale(canvas, proj, rc_chart, map_look.overlay);
+
+  chart.Finish();
 }
 
 void
@@ -294,7 +316,7 @@ FlightStatisticsRenderer::CaptionTask(TCHAR *sTmp, const DerivedInfo &derived)
       !derived.task_stats.total.remaining.IsDefined()) {
     _tcscpy(sTmp, _("No task"));
   } else {
-    const fixed d_remaining = derived.task_stats.total.remaining.GetDistance();
+    const auto d_remaining = derived.task_stats.total.remaining.GetDistance();
     if (task_stats.has_targets) {
       const auto timetext1 = FormatSignedTimeHHMM((int)task_stats.total.time_remaining_start);
       const auto timetext2 = FormatSignedTimeHHMM((int)common.aat_time_remaining);
