@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,11 +22,12 @@ Copyright_License {
 */
 
 #include "HorizonRenderer.hpp"
-#include "Screen/Canvas.hpp"
+#include "ui/canvas/Canvas.hpp"
 #include "Screen/Layout.hpp"
 #include "Look/HorizonLook.hpp"
 #include "NMEA/Attitude.hpp"
-#include "Util/Clamp.hpp"
+#include "Math/Util.hpp"
+#include "util/Clamp.hpp"
 
 #include <algorithm>
 
@@ -47,44 +48,46 @@ HorizonRenderer::Draw(Canvas &canvas, const PixelRect &rc,
   is the case or not.
   */
 
-  const RasterPoint center = rc.GetCenter();
+  const auto center = rc.GetCenter();
 
-  const int radius = std::min(rc.right - rc.left, rc.bottom - rc.top) / 2
+  const int radius = std::min(rc.GetWidth(), rc.GetHeight()) / 2
     - Layout::Scale(1);
 
-  fixed bank_degrees = attitude.IsBankAngleUseable() ?
-                       attitude.bank_angle.Degrees() : fixed(0);
+  auto bank_degrees = attitude.IsBankAngleUseable()
+    ? attitude.bank_angle.Degrees()
+    : 0.;
 
-  fixed pitch_degrees = attitude.IsPitchAngleUseable() ?
-                        attitude.pitch_angle.Degrees() : fixed(0);
+  auto pitch_degrees = attitude.IsPitchAngleUseable()
+    ? attitude.pitch_angle.Degrees()
+    : 0.;
 
-  fixed phi = Clamp(bank_degrees, fixed(-89), fixed(89));
-  Angle alpha = Angle::acos(Clamp(pitch_degrees / 50,
-                                  fixed(-1), fixed(1)));
-  Angle sphi = Angle::HalfCircle() - Angle::Degrees(phi);
-  Angle alpha1 = sphi - alpha;
-  Angle alpha2 = sphi + alpha;
+  auto phi = Clamp(bank_degrees, -89., 89.);
+  auto alpha = Angle::acos(Clamp(pitch_degrees / 50,
+                                 -1., 1.));
+  auto sphi = Angle::HalfCircle() - Angle::Degrees(phi);
+  auto alpha1 = sphi - alpha;
+  auto alpha2 = sphi + alpha;
 
   // draw sky part
   canvas.Select(look.sky_pen);
   canvas.Select(look.sky_brush);
-  canvas.DrawSegment(center.x, center.y, radius, alpha2, alpha1, true);
+  canvas.DrawSegment(center, radius, alpha2, alpha1, true);
 
   // draw ground part
   canvas.Select(look.terrain_pen);
   canvas.Select(look.terrain_brush);
-  canvas.DrawSegment(center.x, center.y, radius, alpha1, alpha2, true);
+  canvas.DrawSegment(center, radius, alpha1, alpha2, true);
 
   // draw aircraft symbol
   canvas.Select(look.aircraft_pen);
-  canvas.DrawLine(center.x + radius / 2, center.y, center.x - radius / 2, center.y);
-  canvas.DrawLine(center.x, center.y - radius / 4, center.x, center.y);
+  canvas.DrawLine({center.x + radius / 2, center.y}, {center.x - radius / 2, center.y});
+  canvas.DrawLine({center.x, center.y - radius / 4}, {center.x, center.y});
 
   // draw 45 degree dash marks
-  const UPixelScalar rr2p = uround(radius * fixed_sqrt_half) + Layout::Scale(1);
-  const UPixelScalar rr2n = rr2p - Layout::Scale(2);
-  canvas.DrawLine(center.x + rr2p, center.y - rr2p,
-              center.x + rr2n, center.y - rr2n);
-  canvas.DrawLine(center.x - rr2p, center.y - rr2p,
-              center.x - rr2n, center.y - rr2n);
+  const int rr2p = uround(radius * M_SQRT1_2) + Layout::Scale(1);
+  const int rr2n = rr2p - Layout::Scale(2);
+  canvas.DrawLine({center.x + rr2p, center.y - rr2p},
+                  {center.x + rr2n, center.y - rr2n});
+  canvas.DrawLine({center.x - rr2p, center.y - rr2p},
+                  {center.x - rr2n, center.y - rr2n});
 }
