@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@ Copyright_License {
 #include "ResourceLoader.hpp"
 #include "ResourceId.hpp"
 
-#include <assert.h>
+#include <cassert>
 
 #ifdef USE_WIN32_RESOURCES
 
@@ -43,7 +43,7 @@ ResourceLoader::Init(HINSTANCE hInstance)
 #else /* !WIN32 */
 
 #include "resource_data.h"
-#include "Util/StringAPI.hxx"
+#include "util/StringAPI.hxx"
 
 #endif /* !WIN32 */
 
@@ -101,15 +101,3 @@ ResourceLoader::LoadBitmap2(ResourceId id)
   return ::LoadBitmap(ResourceLoaderInstance, MAKEINTRESOURCE((unsigned)id));
 }
 #endif
-
-#ifdef HAVE_AYGSHELL_DLL
-#include "OS/AYGShellDLL.hpp"
-
-HBITMAP
-ResourceLoader::SHLoadImageResource(ResourceId id)
-{
-  const AYGShellDLL ayg_shell_dll;
-  return ayg_shell_dll.SHLoadImageResource(ResourceLoaderInstance,
-                                           (unsigned)id);
-}
-#endif
