@@ -159,7 +159,10 @@ SHPTreeHandle msSHPDiskTreeOpen(struct zzip_dir *zdir, const char * pszTree,
     return( NULL );
   }
 
-  zzip_fread( pabyBuf, 8, 1, psTree->fp );
+  if( zzip_fread( pabyBuf, 8, 1, psTree->fp ) != 1 ) {
+    msFree(psTree);
+    return( NULL );
+  }
 
   memcpy( &psTree->signature, pabyBuf, 3 );
   if( strncmp(psTree->signature,"SQT",3) ) {
@@ -195,7 +198,11 @@ SHPTreeHandle msSHPDiskTreeOpen(struct zzip_dir *zdir, const char * pszTree,
     memcpy( &psTree->version, pabyBuf+4, 1 );
     memcpy( &psTree->flags, pabyBuf+5, 3 );
 
-    zzip_fread( pabyBuf, 8, 1, psTree->fp );
+    if( zzip_fread( pabyBuf, 8, 1, psTree->fp ) != 1 )
+    {
+      msFree(psTree);
+      return( NULL );
+    }
   }
 
   if( psTree->needswap ) SwapWord( 4, pabyBuf );
@@ -210,7 +217,7 @@ SHPTreeHandle msSHPDiskTreeOpen(struct zzip_dir *zdir, const char * pszTree,
 
 void msSHPDiskTreeClose(SHPTreeHandle disktree)
 {
-  zzip_fclose( disktree->fp );
+  zzip_file_close( disktree->fp );
   free( disktree );
 }
 
@@ -492,16 +499,19 @@ static void searchDiskTreeNode(SHPTreeHandle disktree, rectObj aoi, ms_bitarray
 
   int *ids=NULL;
 
-  zzip_fread( &offset, 4, 1, disktree->fp );
+  if( zzip_fread( &offset, 4, 1, disktree->fp ) != 1 )
+    goto error;
   if ( disktree->needswap ) SwapWord ( 4, &offset );
 
-  zzip_fread( &rect, sizeof(rectObj), 1, disktree->fp );
+  if( zzip_fread( &rect, sizeof(rectObj), 1, disktree->fp ) != 1 )
+    goto error;
   if ( disktree->needswap ) SwapWord ( 8, &rect.minx );
   if ( disktree->needswap ) SwapWord ( 8, &rect.miny );
   if ( disktree->needswap ) SwapWord ( 8, &rect.maxx );
   if ( disktree->needswap ) SwapWord ( 8, &rect.maxy );
 
-  zzip_fread( &numshapes, 4, 1, disktree->fp );
+  if( zzip_fread( &numshapes, 4, 1, disktree->fp ) != 1 )
+    goto error;
   if ( disktree->needswap ) SwapWord ( 4, &numshapes );
 
   if(!msRectOverlap(&rect, &aoi)) { /* skip rest of this node and sub-nodes */
@@ -512,7 +522,8 @@ static void searchDiskTreeNode(SHPTreeHandle disktree, rectObj aoi, ms_bitarray
   if(numshapes > 0) {
     ids = (int *)msSmallMalloc(numshapes*sizeof(ms_int32));
 
-    zzip_fread( ids, numshapes*sizeof(ms_int32), 1, disktree->fp );
+    if( zzip_fread( ids, numshapes*sizeof(ms_int32), 1, disktree->fp ) != 1 )
+      goto error;
     if (disktree->needswap ) {
       for( i=0; i<numshapes; i++ ) {
         SwapWord( 4, &ids[i] );
@@ -525,16 +536,21 @@ static void searchDiskTreeNode(SHPTreeHandle disktree, rectObj aoi, ms_bitarray
     free(ids);
   }
 
-  zzip_fread( &numsubnodes, 4, 1, disktree->fp );
+  if( zzip_fread( &numsubnodes, 4, 1, disktree->fp ) != 1 )
+    goto error;
   if ( disktree->needswap ) SwapWord ( 4, &numsubnodes );
 
   for(i=0; i<numsubnodes; i++)
     searchDiskTreeNode(disktree, aoi, status);
 
   return;
+  
+error:
+  msSetError(MS_IOERR, NULL, "searchDiskTreeNode()");
+  return;
 }
 
-ms_bitarray msSearchDiskTree(struct zzip_dir *zdir, const char *filename, rectObj aoi, int debug)
+ms_bitarray msSearchDiskTree(struct zzip_dir *zdir, const char *filename, rectObj aoi, int debug, int numshapes)
 {
   SHPTreeHandle disktree;
   ms_bitarray status=NULL;
@@ -548,6 +564,12 @@ ms_bitarray msSearchDiskTree(struct zzip_dir *zdir, const char *filename, rectOb
     return(NULL);
   }
 
+  if (disktree->nShapes != numshapes) {
+      msSetError(MS_SHPERR, "The spatial index file %s is corrupt.", "msSearchDiskTree()", filename);
+      msSHPDiskTreeClose(disktree);
+      return(NULL);
+  }
+
   status = msAllocBitArray(disktree->nShapes);
   if(!status) {
     msSetError(MS_MEMERR, NULL, "msSearchDiskTree()");
@@ -574,26 +596,51 @@ treeNodeObj *readTreeNode( SHPTreeHandle disktree )
 
   res = fread( &offset, 4, 1, disktree->fp );
   if ( !res )
+  {
+    free(node);
     return NULL;
+  }
 
   if ( disktree->needswap ) SwapWord ( 4, &offset );
 
-  fread( &node->rect, sizeof(rectObj), 1, disktree->fp );
+  res = fread( &node->rect, sizeof(rectObj), 1, disktree->fp );
+  if ( !res )
+  {
+    free(node);
+    return NULL;
+  }
   if ( disktree->needswap ) SwapWord ( 8, &node->rect.minx );
   if ( disktree->needswap ) SwapWord ( 8, &node->rect.miny );
   if ( disktree->needswap ) SwapWord ( 8, &node->rect.maxx );
   if ( disktree->needswap ) SwapWord ( 8, &node->rect.maxy );
 
-  fread( &node->numshapes, 4, 1, disktree->fp );
+  res = fread( &node->numshapes, 4, 1, disktree->fp );
+  if ( !res )
+  {
+    free(node);
+    return NULL;
+  }
   if ( disktree->needswap ) SwapWord ( 4, &node->numshapes );
   if( node->numshapes > 0 )
     node->ids = (ms_int32 *)msSmallMalloc(sizeof(ms_int32)*node->numshapes);
-  fread( node->ids, node->numshapes*4, 1, disktree->fp );
+  res = fread( node->ids, node->numshapes*4, 1, disktree->fp );
+  if ( !res )
+  {
+    free(node->ids);
+    free(node);
+    return NULL;
+  }
   for( i=0; i < node->numshapes; i++ ) {
     if ( disktree->needswap ) SwapWord ( 4, &node->ids[i] );
   }
 
-  fread( &node->numsubnodes, 4, 1, disktree->fp );
+  res = fread( &node->numsubnodes, 4, 1, disktree->fp );
+  if ( !res )
+  {
+    free(node->ids);
+    free(node);
+    return NULL;
+  }
   if ( disktree->needswap ) SwapWord ( 4, &node->numsubnodes );
 
   return node;
