@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,7 +24,8 @@ Copyright_License {
 #include "Units/UnitsGlue.hpp"
 #include "Units/UnitsStore.hpp"
 #include "LogFile.hpp"
-#include "Util/StringAPI.hxx"
+#include "util/ScopeExit.hxx"
+#include "util/StringAPI.hxx"
 
 #include <tchar.h>
 
@@ -32,14 +33,11 @@ Copyright_License {
 #include <windows.h>
 #endif
 
-#ifdef _WIN32_WCE
-#include "OS/DynamicLibrary.hpp"
-#endif
-
 #ifdef ANDROID
-#include "Java/Global.hxx"
-#include "Java/Class.hxx"
-#include "Java/Object.hxx"
+#include "java/Global.hxx"
+#include "java/Class.hxx"
+#include "java/Object.hxx"
+#include "java/String.hxx"
 #endif
 
 struct language_unit_map {
@@ -107,25 +105,6 @@ AutoDetect()
 {
 #ifndef HAVE_POSIX
 
-#if defined(_WIN32_WCE)
-  /* the GetUserDefaultUILanguage() prototype is missing on
-     mingw32ce, we have to look it up dynamically */
-  DynamicLibrary coreloc_dll(_T("coredll"));
-  if (!coreloc_dll.IsDefined()) {
-    LogFormat("Units: coredll.dll not found");
-    return 0;
-  }
-
-  typedef LANGID WINAPI (*GetUserDefaultUILanguage_t)();
-  GetUserDefaultUILanguage_t GetUserDefaultUILanguage =
-    (GetUserDefaultUILanguage_t)
-    coreloc_dll.Lookup(_T("GetUserDefaultUILanguage"));
-  if (GetUserDefaultUILanguage == nullptr) {
-    LogFormat("Units: GetUserDefaultUILanguage() not available");
-    return 0;
-  }
-#endif
-
   // Retrieve the default user language identifier from the OS
   LANGID lang_id = GetUserDefaultUILanguage();
   LogFormat("Units: GetUserDefaultUILanguage() = 0x%x", (int)lang_id);
@@ -146,37 +125,27 @@ AutoDetect()
                                          "()Ljava/util/Locale;");
   assert(cid != nullptr);
 
-  jobject _obj = env->CallStaticObjectMethod(cls, cid);
-  if (_obj == nullptr)
+  Java::LocalObject obj(env, env->CallStaticObjectMethod(cls, cid));
+  if (!obj)
     return 0;
 
-  Java::LocalObject obj(env, _obj);
-
   // Call function Locale.getLanguage() that
   // returns a two-letter language string
 
-  cid = env->GetMethodID(cls, "toString", "()Ljava/lang/String;");
-  assert(cid != nullptr);
-
-  jstring language = (jstring)env->CallObjectMethod(obj, cid);
+  Java::String language{env, Java::Object::toString(env, obj)};
   if (language == nullptr)
     return 0;
 
   // Convert the jstring to a char string
   const char *language2 = env->GetStringUTFChars(language, nullptr);
-  if (language2 == nullptr) {
-    env->DeleteLocalRef(language);
+  if (language2 == nullptr)
     return 0;
-  }
 
-  unsigned id = FindLanguage(language2);
+  AtScopeExit(env, &language, language2) {
+    env->ReleaseStringUTFChars(language, language2);
+  };
 
-  // Clean up the memory
-  env->ReleaseStringUTFChars(language, language2);
-  env->DeleteLocalRef(language);
-
-  // Return e.g. "de.mo"
-  return id;
+  return FindLanguage(language2);
 
 #else
   // Metric default on Linux
