@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -29,27 +29,25 @@ Copyright_License {
 #include "Startup.hpp"
 #include "LocalPath.hpp"
 #include "Version.hpp"
-#include "Protection.hpp"
-#include "Components.hpp"
 #include "LogFile.hpp"
 #include "CommandLine.hpp"
 #include "MainWindow.hpp"
 #include "Interface.hpp"
-#include "Compiler.h"
 #include "Look/GlobalFonts.hpp"
-#include "Screen/Init.hpp"
-#include "Net/HTTP/Init.hpp"
+#include "ui/window/Init.hpp"
+#include "net/http/Init.hpp"
 #include "UtilsSystem.hpp"
 #include "ResourceLoader.hpp"
 #include "Language/Language.hpp"
 #include "Language/LanguageGlue.hpp"
 #include "Simulator.hpp"
-#include "OS/Args.hpp"
-#include "IO/Async/GlobalIOThread.hpp"
-
-#ifndef NDEBUG
-#include "Thread/Thread.hpp"
-#endif
+#include "Audio/GlobalPCMMixer.hpp"
+#include "Audio/GlobalPCMResourcePlayer.hpp"
+#include "Audio/GlobalVolumeController.hpp"
+#include "system/Args.hpp"
+#include "io/async/GlobalAsioThread.hpp"
+#include "io/async/AsioThread.hpp"
+#include "util/PrintException.hxx"
 
 #ifdef ENABLE_SDL
 /* this is necessary on Mac OS X, to let libSDL bootstrap Quartz
@@ -64,7 +62,7 @@ Copyright_License {
 #endif
 #endif
 
-#include <assert.h>
+#include <cassert>
 
 static const char *const Usage = "\n"
   "  -datapath=      path to XCSoar data can be defined\n"
@@ -73,13 +71,11 @@ static const char *const Usage = "\n"
   "  -fly            bypass startup-screen, use fly mode directly\n"
 #endif
   "  -profile=fname  load profile from file fname\n"
-#if !defined(_WIN32_WCE)
   "  -WIDTHxHEIGHT   use screen resolution WIDTH x HEIGHT\n"
   "  -portrait       use a 480x640 screen resolution\n"
   "  -square         use a 480x480 screen resolution\n"
   "  -small          use a 320x240 screen resolution\n"
-#endif
-#if !defined(ANDROID) && !defined(_WIN32_WCE)
+#if !defined(ANDROID)
   "  -dpi=DPI        force usage of DPI for pixel density\n"
   "  -dpi=XDPIxYDPI  force usage of XDPI and YDPI for pixel density\n"
 #endif
@@ -89,24 +85,61 @@ static const char *const Usage = "\n"
 #ifdef HAVE_CMDLINE_RESIZABLE
   "  -resizable      resizable window\n"
 #endif
-#if defined(_WIN32) && !defined(_WIN32_WCE)
+#ifdef _WIN32
   "  -console        open debug output console\n"
 #endif
   ;
 
+static int
+Main()
+{
+  ScreenGlobalInit screen_init;
+
+#if defined(__APPLE__) && !TARGET_OS_IPHONE
+  // We do not want the ugly non-localized main menu which SDL creates
+  [NSApp setMainMenu: [[NSMenu alloc] init]];
+#endif
+
+#ifdef _WIN32
+  /* try to make the UI most responsive */
+  SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
+#endif
+
+  AllowLanguage();
+  InitLanguage();
+
+  ScopeGlobalAsioThread global_asio_thread;
+  const Net::ScopeInit net_init(asio_thread->GetEventLoop());
+
+  ScopeGlobalPCMMixer global_pcm_mixer(asio_thread->GetEventLoop());
+  ScopeGlobalPCMResourcePlayer global_pcm_resouce_player;
+  ScopeGlobalVolumeController global_volume_controller;
+
+  // Perform application initialization and run loop
+  int ret = EXIT_FAILURE;
+  if (Startup())
+    ret = CommonInterface::main_window->RunEventLoop();
+
+  Shutdown();
+
+  DisallowLanguage();
+
+  Fonts::Deinitialize();
+
+  DeinitialiseDataPath();
+
+  return ret;
+}
+
 /**
  * Main entry point for the whole XCSoar application
  */
-#ifndef WIN32
+#ifndef _WIN32
 int main(int argc, char **argv)
 #else
 int WINAPI
 WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
-#ifdef _WIN32_WCE
-        gcc_unused LPWSTR lpCmdLine,
-#else
         gcc_unused LPSTR lpCmdLine2,
-#endif
         int nCmdShow)
 #endif
 {
@@ -114,8 +147,6 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   ResourceLoader::Init(hInstance);
 #endif
 
-  Net::Initialise();
-
   InitialiseDataPath();
   StartupLogFreeRamAndStorage();
 
@@ -124,7 +155,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
   // Read options from the command line
   {
-#ifdef WIN32
+#ifdef _WIN32
     Args args(GetCommandLine(), Usage);
 #else
     Args args(argc, argv, Usage);
@@ -132,40 +163,14 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
     CommandLine::Parse(args);
   }
 
-  ScreenGlobalInit screen_init;
+  int ret;
 
-#if defined(__APPLE__) && !TARGET_OS_IPHONE
-  // We do not want the ugly non-localized main menu which SDL creates
-  [NSApp setMainMenu: [[NSMenu alloc] init]];
-#endif
-
-#ifdef WIN32
-  /* try to make the UI most responsive */
-  SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
-#endif
-
-  AllowLanguage();
-  InitLanguage();
-
-  InitialiseIOThread();
-
-  // Perform application initialization and run loop
-  int ret = EXIT_FAILURE;
-  if (Startup())
-    ret = CommonInterface::main_window->RunEventLoop();
-
-  Shutdown();
-
-  DeinitialiseIOThread();
-
-  DisallowLanguage();
-
-  Fonts::Deinitialize();
-
-  DeinitialiseDataPath();
-  Net::Deinitialise();
-
-  assert(!ExistsAnyThread());
+  try {
+    ret = Main();
+  } catch (...) {
+    PrintException(std::current_exception());
+    ret = EXIT_FAILURE;
+  }
 
 #if defined(__APPLE__) && TARGET_OS_IPHONE
   /* For some reason, the app process does not exit on iOS, but a black
