@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,6 +22,7 @@
 
 #include "Device/Driver/Generic.hpp"
 #include "Device/Driver/AltairPro.hpp"
+#include "Device/Driver/AirControlDisplay.hpp"
 #include "Device/Driver/BlueFlyVario.hpp"
 #include "Device/Driver/BorgeltB50.hpp"
 #include "Device/Driver/CAI302.hpp"
@@ -46,6 +47,7 @@
 #include "Device/Driver/Vega.hpp"
 #include "Device/Driver/Volkslogger.hpp"
 #include "Device/Driver/Westerboer.hpp"
+#include "Device/Driver/XCTracer.hpp"
 #include "Device/Driver/Zander.hpp"
 #include "Device/Driver.hpp"
 #include "Device/RecordedFlight.hpp"
@@ -64,6 +66,7 @@
 #include "FaultInjectionPort.hpp"
 #include "TestUtil.hpp"
 #include "Units/System.hpp"
+#include "io/NullDataHandler.hpp"
 
 #include <memory>
 
@@ -80,7 +83,7 @@ TestGeneric()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   nmea_info.alive.Update(nmea_info.clock);
 
   /* no GPS reception */
@@ -124,6 +127,11 @@ TestGeneric()
 
   /* Magnetic Heading bad checksum */
   ok1(!parser.ParseLine("$HCHDM,182.7,M*26", nmea_info));
+
+  ok1(parser.ParseLine("$WIMWV,12.1,T,10.1,M,A*24", nmea_info));
+  ok1(nmea_info.external_wind_available);
+  ok1(equals(nmea_info.external_wind.bearing, 12.1));
+  ok1(nmea_info.external_wind.norm == 10.1);
 }
 
 static void
@@ -133,23 +141,23 @@ TestTasman()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(parser.ParseLine("$PTAS1,200,200,02426,000*25", nmea_info));
   ok1(nmea_info.total_energy_vario_available);
-  ok1(equals(nmea_info.total_energy_vario, fixed(0)));
+  ok1(equals(nmea_info.total_energy_vario, 0));
   ok1(nmea_info.pressure_altitude_available);
-  ok1(equals(nmea_info.pressure_altitude, Units::ToSysUnit(fixed(426), Unit::FEET)));
+  ok1(equals(nmea_info.pressure_altitude, Units::ToSysUnit(426, Unit::FEET)));
   ok1(nmea_info.airspeed_available);
-  ok1(equals(nmea_info.true_airspeed, fixed(0)));
+  ok1(equals(nmea_info.true_airspeed, 0));
 
   ok1(parser.ParseLine("$PTAS1,234,000,00426,062*26", nmea_info));
   ok1(nmea_info.total_energy_vario_available);
-  ok1(equals(nmea_info.total_energy_vario, Units::ToSysUnit(fixed(3.4), Unit::KNOTS)));
+  ok1(equals(nmea_info.total_energy_vario, Units::ToSysUnit(3.4, Unit::KNOTS)));
   ok1(nmea_info.pressure_altitude_available);
-  ok1(equals(nmea_info.pressure_altitude, Units::ToSysUnit(fixed(-1574), Unit::FEET)));
+  ok1(equals(nmea_info.pressure_altitude, Units::ToSysUnit(-1574, Unit::FEET)));
   ok1(nmea_info.airspeed_available);
-  ok1(equals(nmea_info.true_airspeed, Units::ToSysUnit(fixed(62), Unit::KNOTS)));
+  ok1(equals(nmea_info.true_airspeed, Units::ToSysUnit(62, Unit::KNOTS)));
 }
 
 static void
@@ -159,7 +167,7 @@ TestFLARM()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(parser.ParseLine("$PFLAU,3,1,1,1,0*50",
                                       nmea_info));
@@ -255,7 +263,7 @@ TestAltairRU()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PTFRS,1,0,0,0,0,0,0,0,5,1,10,0,3,1338313437,0,0,0,,,2*4E",
                         nmea_info));
@@ -265,7 +273,7 @@ TestAltairRU()
   ok1(!nmea_info.voltage_available);
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PTFRS,1,0,0,0,0,0,0,0,342,1,10,0,3,1338313438,0,0,12743,,,2*42",
                         nmea_info));
@@ -285,7 +293,7 @@ TestBlueFly()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   // repeat input to get stable filter output
   ok1(device->ParseNMEA("PRS 00017CBA", nmea_info));
@@ -298,7 +306,7 @@ TestBlueFly()
   ok1(equals(nmea_info.static_pressure.GetPascal(), 97466));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("PRS 00017CCA", nmea_info));
   ok1(device->ParseNMEA("PRS 00017CCA", nmea_info));
@@ -317,7 +325,7 @@ TestBlueFly()
   ok1(equals(nmea_info.battery_level, 50.0));
   ok1(device->ParseNMEA("BAT ED8", nmea_info)); //3.80V
   ok1(nmea_info.battery_level_available);
-  ok1(equals(nmea_info.battery_level, 37.0));
+  ok1(equals(nmea_info.battery_level, 37.3333));
 
   delete device;
 }
@@ -331,7 +339,7 @@ TestBorgeltB50()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PBB50,042,-01.1,1.0,12345,10,1.3,1,-28*75", nmea_info));
   ok1(nmea_info.airspeed_available);
@@ -347,7 +355,7 @@ TestBorgeltB50()
   ok1(equals(nmea_info.settings.ballast_overload, 1.3));
   ok1(nmea_info.switch_state.flight_mode == SwitchState::FlightMode::CIRCLING);
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature, 245.15));
+  ok1(equals(nmea_info.temperature.ToKelvin(), 245.15));
 
   delete device;
 }
@@ -361,7 +369,7 @@ TestCAI302()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("!w,000,000,0000,500,01287,01020,-0668,191,199,191,000,000,100*44",
                         nmea_info));
@@ -419,7 +427,7 @@ TestCProbe()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PCPROBE,T,FD92,FF93,00D9,FD18,017E,FEDB,0370,0075,00D6,0064,001C,000000,,",
                         nmea_info));
@@ -433,8 +441,8 @@ TestCProbe()
   ok1(nmea_info.acceleration.real);
   ok1(equals(nmea_info.acceleration.g_load, 1.0030817514));
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature,
-             Units::ToSysUnit(fixed(11.7), Unit::DEGREES_CELCIUS)));
+  ok1(equals(nmea_info.temperature.ToKelvin(),
+             Temperature::FromCelsius(11.7).ToKelvin()));
   ok1(nmea_info.humidity_available);
   ok1(equals(nmea_info.humidity, 21.4));
   ok1(nmea_info.battery_level_available);
@@ -456,7 +464,7 @@ TestEye()
 
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PEYA,1015.5,1020.5,3499,1012.3,265,12,176,+05.4,+15.2,095,1650,+05.1,+3.9*3a",
                         nmea_info));
@@ -471,22 +479,22 @@ TestEye()
   ok1(nmea_info.external_wind_available);
   ok1(equals(nmea_info.external_wind.bearing, 265));
   ok1(equals(nmea_info.external_wind.norm,
-             Units::ToSysUnit(fixed(12), Unit::KILOMETER_PER_HOUR)));
+             Units::ToSysUnit(12, Unit::KILOMETER_PER_HOUR)));
   ok1(nmea_info.airspeed_available);
   ok1(nmea_info.airspeed_real);
   ok1(equals(nmea_info.true_airspeed,
-             Units::ToSysUnit(fixed(176), Unit::KILOMETER_PER_HOUR)));
+             Units::ToSysUnit(176, Unit::KILOMETER_PER_HOUR)));
   ok1(nmea_info.noncomp_vario_available);
   ok1(equals(nmea_info.noncomp_vario, 5.4));
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature,
-             Units::ToSysUnit(fixed(15.2), Unit::DEGREES_CELCIUS)));
+  ok1(equals(nmea_info.temperature.ToKelvin(),
+             Temperature::FromCelsius(15.2).ToKelvin()));
   ok1(nmea_info.humidity_available);
   ok1(equals(nmea_info.humidity, 95));
 
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PEYI,+110,+020,+135,+130,+140,+0.12,+1.03,+9.81,+12,248,246,+02.3,*16",
                         nmea_info));
@@ -510,7 +518,7 @@ TestFlymasterF1()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$VARIO,999.98,-12,12.4,12.7,0,21.3,25.5*66",
                         nmea_info));
@@ -519,8 +527,8 @@ TestFlymasterF1()
   ok1(equals(nmea_info.total_energy_vario, -1.2));
   ok1(!nmea_info.voltage_available);
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature,
-             Units::ToSysUnit(fixed(21.3), Unit::DEGREES_CELCIUS)));
+  ok1(equals(nmea_info.temperature.ToKelvin(),
+             Temperature::FromCelsius(21.3).ToKelvin()));
   ok1(!nmea_info.baro_altitude_available);
   ok1(!nmea_info.pressure_altitude_available);
   ok1(nmea_info.static_pressure_available);
@@ -548,35 +556,35 @@ TestFlyNet()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("_PRS 00017CBA", nmea_info));
   ok1(nmea_info.static_pressure_available);
   ok1(equals(nmea_info.static_pressure.GetPascal(), 97466));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("_PRS 00018BCD", nmea_info));
   ok1(nmea_info.static_pressure_available);
   ok1(equals(nmea_info.static_pressure.GetPascal(), 101325));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("_BAT 0", nmea_info));
   ok1(nmea_info.battery_level_available);
   ok1(equals(nmea_info.battery_level, 0));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("_BAT 7", nmea_info));
   ok1(nmea_info.battery_level_available);
   ok1(equals(nmea_info.battery_level, 70));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("_BAT A", nmea_info));
   ok1(nmea_info.battery_level_available);
@@ -594,7 +602,7 @@ TestFlytec()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$BRSF,063,-013,-0035,1,193,00351,535,485*33",
                         nmea_info));
@@ -602,7 +610,7 @@ TestFlytec()
   ok1(equals(nmea_info.true_airspeed, 17.5));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$VMVABD,1234.5,M,0547.0,M,-0.0,,,MS,63.0,KH,22.4,C*51",
                         nmea_info));
@@ -613,10 +621,10 @@ TestFlytec()
   ok1(nmea_info.airspeed_available);
   ok1(equals(nmea_info.true_airspeed, 17.5));
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature, 295.55));
+  ok1(equals(nmea_info.temperature.ToKelvin(), 295.55));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$FLYSEN,,,,,,,,,V,,101450,02341,0334,02000,,,,,,,,,*72",
                         nmea_info));
@@ -632,7 +640,7 @@ TestFlytec()
   ok1(equals(nmea_info.true_airspeed, 200));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$FLYSEN,,,,,,,,,,V,,101450,02341,0334,02000,,,,,,,,,*5e",
                         nmea_info));
@@ -652,7 +660,7 @@ TestFlytec()
   ok1(!device->ParseNMEA("$FLYSEN,,,,,,,,,,,,,,,,,,,,*5e", nmea_info));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$FLYSEN,241211,201500,4700.840,N,00818.457,E,092,"
                         "01100,01234,A,09,097517,01321,-001,01030,P,023,,038,"
@@ -686,10 +694,11 @@ TestFlytec()
   ok1(nmea_info.battery_level_available);
   ok1(equals(nmea_info.battery_level, (88.0 + 38.0) / 2));
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature, Units::ToSysUnit(fixed(23), Unit::DEGREES_CELCIUS)));
+  ok1(equals(nmea_info.temperature.ToKelvin(),
+             Temperature::FromCelsius(23).ToKelvin()));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$FLYSEN,241211,201500,4700.840,N,00818.457,E,092,"
                         "01100,01234,V,09,097517,01321,-001,01030,P,023,017,038,"
@@ -719,7 +728,8 @@ TestFlytec()
   ok1(nmea_info.battery_level_available);
   ok1(equals(nmea_info.battery_level, 38.0));
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature, Units::ToSysUnit(fixed(17), Unit::DEGREES_CELCIUS)));
+  ok1(equals(nmea_info.temperature.ToKelvin(),
+             Temperature::FromCelsius(17).ToKelvin()));
 
   delete device;
 }
@@ -733,7 +743,7 @@ TestLeonardo()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$C,+2025,-7,+18,+25,+29,122,314,314,0,-356,+25,45,T*3D",
                         nmea_info));
@@ -747,14 +757,14 @@ TestLeonardo()
   ok1(equals(nmea_info.netto_vario, 2.5));
 
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature, 302.15));
+  ok1(equals(nmea_info.temperature.ToKelvin(), 302.15));
 
   ok1(nmea_info.external_wind_available);
   ok1(equals(nmea_info.external_wind.bearing, 45));
   ok1(equals(nmea_info.external_wind.norm, 6.94444444));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$c,+2025,-2,+18*5C", nmea_info));
   ok1(nmea_info.baro_altitude_available);
@@ -763,11 +773,11 @@ TestLeonardo()
   ok1(equals(nmea_info.total_energy_vario, -0.02));
   ok1(nmea_info.airspeed_available);
   ok1(equals(nmea_info.true_airspeed,
-             Units::ToSysUnit(fixed(18), Unit::KILOMETER_PER_HOUR)));
+             Units::ToSysUnit(18, Unit::KILOMETER_PER_HOUR)));
   ok1(!nmea_info.netto_vario_available);
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$D,+7,100554,+25,18,+31,,0,-356,+25,+11,115,96*6A",
                         nmea_info));
@@ -780,10 +790,10 @@ TestLeonardo()
   ok1(nmea_info.airspeed_available);
   ok1(equals(nmea_info.true_airspeed, 5));
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature, 304.15));
+  ok1(equals(nmea_info.temperature.ToKelvin(), 304.15));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PDGFTL1,2025,2000,250,-14,45,134,28,65,382,153*3D",
                         nmea_info));
@@ -798,7 +808,7 @@ TestLeonardo()
   ok1(nmea_info.airspeed_available);
   ok1(nmea_info.airspeed_real);
   ok1(equals(nmea_info.indicated_airspeed,
-             Units::ToSysUnit(fixed(45), Unit::KILOMETER_PER_HOUR)));
+             Units::ToSysUnit(45, Unit::KILOMETER_PER_HOUR)));
   ok1(nmea_info.external_wind_available);
   ok1(equals(nmea_info.external_wind.bearing, 65));
   ok1(equals(nmea_info.external_wind.norm, 7.777777));
@@ -806,7 +816,7 @@ TestLeonardo()
   ok1(equals(nmea_info.voltage, 3.82));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PDGFTTL,2025,2000,250,-14*41", nmea_info));
   ok1(nmea_info.pressure_altitude_available);
@@ -830,7 +840,7 @@ TestLevilAHRS()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   // All angles in tenth of degrees
   ok1(device->ParseNMEA("$RPYL,127,729,3215,99,88,1376,0,", nmea_info));
@@ -865,7 +875,7 @@ TestLX(const struct DeviceRegister &driver, bool condor=false)
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   /* empty sentence */
   ok1(device->ParseNMEA("$LXWP0,N,,,,,,,,,,,*6d", nmea_info));
@@ -877,7 +887,7 @@ TestLX(const struct DeviceRegister &driver, bool condor=false)
 
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   /* altitude and wind */
   ok1(device->ParseNMEA("$LXWP0,N,,1266.5,,,,,,,,248,23.1*55", nmea_info));
@@ -901,7 +911,7 @@ TestLX(const struct DeviceRegister &driver, bool condor=false)
 
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   /* airspeed and vario available */
   ok1(device->ParseNMEA("$LXWP0,Y,222.3,1665.5,1.71,,,,,,239,174,10.1*47",
@@ -921,7 +931,7 @@ TestLX(const struct DeviceRegister &driver, bool condor=false)
 
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   /* airspeed without altitude */
   ok1(device->ParseNMEA("$LXWP0,Y,222.3,,,,,,,,,,*55",
@@ -950,7 +960,7 @@ TestLX(const struct DeviceRegister &driver, bool condor=false)
 
 
     nmea_info.Reset();
-    nmea_info.clock = fixed(1);
+    nmea_info.clock = 1;
 
     // Test LX160 (sw 3.04) variant (different bugs notation)
     ok1(device->ParseNMEA("$LXWP2,1.1,1.00,1.00,2.14,-3.87,2.38*3E", nmea_info));
@@ -991,7 +1001,7 @@ TestLX(const struct DeviceRegister &driver, bool condor=false)
 
 
     nmea_info.Reset();
-    nmea_info.clock = fixed(1);
+    nmea_info.clock = 1;
 
     LXDevice &lx_device = *(LXDevice *)device;
     ok1(!lx_device.IsV7());
@@ -1039,7 +1049,7 @@ TestLXV7()
 
   NMEAInfo basic;
   basic.Reset();
-  basic.clock = fixed(1);
+  basic.clock = 1;
 
   LXDevice &lx_device = *(LXDevice *)device;
   lx_device.ResetDeviceDetection();
@@ -1057,7 +1067,7 @@ TestLXV7()
 
   ok1(device->ParseNMEA("$PLXVS,23.1,0,12.3,*71", basic));
   ok1(basic.temperature_available);
-  ok1(equals(basic.temperature, 296.25));
+  ok1(equals(basic.temperature.ToKelvin(), 296.25));
   ok1(basic.switch_state.flight_mode == SwitchState::FlightMode::CIRCLING);
   ok1(basic.voltage_available);
   ok1(equals(basic.voltage, 12.3));
@@ -1076,7 +1086,7 @@ TestILEC()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   /* baro altitude disabled */
   ok1(device->ParseNMEA("$PILC,PDA1,1489,-3.21*69", nmea_info));
@@ -1107,7 +1117,7 @@ TestVega()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   /* enable FLARM mode (switches the $PGRMZ parser to pressure
      altitude) */
@@ -1151,7 +1161,7 @@ TestOpenVario()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   // Empty sentence is handled by device driver
   ok1(device->ParseNMEA("$POV*49", nmea_info));
@@ -1203,9 +1213,15 @@ TestOpenVario()
   // Temperature is read
   ok1(device->ParseNMEA("$POV,T,23.52*35", nmea_info));
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature, 23.52 + 273.15));
+  ok1(equals(nmea_info.temperature.ToKelvin(),
+             Temperature::FromCelsius(23.52).ToKelvin()));
   nmea_info.Reset();
 
+  // Relative humidity is read
+  ok1(device->ParseNMEA("$POV,H,58.42*24", nmea_info));
+  ok1(nmea_info.humidity_available);
+  ok1(equals(nmea_info.humidity, 58.42));
+
   delete device;
 }
 
@@ -1218,7 +1234,7 @@ TestWesterboer()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PWES0,20,-25,25,-22,2,-100,589,589,1260,1296,128,295*01",
                         nmea_info));
@@ -1234,7 +1250,8 @@ TestWesterboer()
   ok1(nmea_info.voltage_available);
   ok1(equals(nmea_info.voltage, 12.8));
   ok1(nmea_info.temperature_available);
-  ok1(equals(nmea_info.temperature, 29.5 + 273.15));
+  ok1(equals(nmea_info.temperature.ToKelvin(),
+             Temperature::FromCelsius(29.5).ToKelvin()));
 
   ok1(device->ParseNMEA("$PWES1,20,21,0,030,1,6,385,10*1a", nmea_info));
   ok1(nmea_info.settings.mac_cready_available);
@@ -1256,7 +1273,7 @@ TestZander()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   /* baro altitude enabled */
   ok1(device->ParseNMEA("$PZAN1,02476,123456*04", nmea_info));
@@ -1265,57 +1282,57 @@ TestZander()
 
   ok1(device->ParseNMEA("$PZAN2,123,9850*03", nmea_info));
   ok1(nmea_info.airspeed_available);
-  ok1(equals(nmea_info.true_airspeed, fixed(34.1667)));
+  ok1(equals(nmea_info.true_airspeed, 34.1667));
   ok1(nmea_info.total_energy_vario_available);
-  ok1(equals(nmea_info.total_energy_vario, fixed(-1.5)));
+  ok1(equals(nmea_info.total_energy_vario, -1.5));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN3,+,026,V,321,035,A,321,035,V*44", nmea_info));
   ok1(nmea_info.external_wind_available);
   ok1(equals(nmea_info.external_wind.bearing, 321));
   ok1(equals(nmea_info.external_wind.norm, 9.72222));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN3,+,026,V,321,035,V,321,035,V*53", nmea_info));
   ok1(!nmea_info.external_wind_available);
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN3,+,026,A,321,035,A*2f", nmea_info));
   ok1(nmea_info.external_wind_available);
   ok1(equals(nmea_info.external_wind.bearing, 321));
   ok1(equals(nmea_info.external_wind.norm, 9.72222));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN3,+,026,A,321,035,A,V*55", nmea_info));
   ok1(nmea_info.external_wind_available);
   ok1(equals(nmea_info.external_wind.bearing, 321));
   ok1(equals(nmea_info.external_wind.norm, 9.72222));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN3,+,026,A,321,035,V,A*55", nmea_info));
   ok1(nmea_info.external_wind_available);
   ok1(equals(nmea_info.external_wind.bearing, 321));
   ok1(equals(nmea_info.external_wind.norm, 9.72222));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN3,+,026,A,321,035,A,A*42", nmea_info));
   ok1(nmea_info.external_wind_available);
   ok1(equals(nmea_info.external_wind.bearing, 321));
   ok1(equals(nmea_info.external_wind.norm, 9.72222));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN3,+,026,A,321,035,V*38", nmea_info));
   ok1(!nmea_info.external_wind_available);
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN3,+,026,A,321,035,V,V*42", nmea_info));
   ok1(!nmea_info.external_wind_available);
 
@@ -1324,17 +1341,17 @@ TestZander()
   ok1(equals(nmea_info.settings.mac_cready, 1.5));
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN5,,MUEHL,123.4,KM,T,234*24", nmea_info));
   ok1(nmea_info.switch_state.flight_mode == SwitchState::FlightMode::UNKNOWN);
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN5,SF,MUEHL,123.4,KM,T,234*31", nmea_info));
   ok1(nmea_info.switch_state.flight_mode == SwitchState::FlightMode::CRUISE);
 
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
   ok1(device->ParseNMEA("$PZAN5,VA,MUEHL,123.4,KM,T,234*33", nmea_info));
   ok1(nmea_info.switch_state.flight_mode == SwitchState::FlightMode::CIRCLING);
 
@@ -1350,7 +1367,7 @@ TestVaulter()
 
   NMEAInfo nmea_info;
   nmea_info.Reset();
-  nmea_info.clock = fixed(1);
+  nmea_info.clock = 1;
 
   ok1(device->ParseNMEA("$PITV3,-16.0,-23.9,147.9,23.03,1.01*1C", nmea_info));
   ok1(nmea_info.attitude.bank_angle_available);
@@ -1382,11 +1399,143 @@ TestVaulter()
   delete device;
 }
 
+static void
+TestXCTracer()
+{
+  NullPort null;
+  Device *device = xctracer_driver.CreateOnPort(dummy_config, null);
+  ok1(device != NULL);
+
+  NMEAInfo nmea_info;
+  nmea_info.Reset();
+  nmea_info.clock = 1;
+
+  /* empty sentence */
+  ok1(device->ParseNMEA("$LXWP0,N,,,,,,,,,,,*6d", nmea_info));
+  ok1(!nmea_info.pressure_altitude_available);
+  ok1(!nmea_info.baro_altitude_available);
+  ok1(!nmea_info.airspeed_available);
+  ok1(!nmea_info.total_energy_vario_available);
+  ok1(!nmea_info.external_wind_available);
+
+  /* bad checksum */
+  ok1(!device->ParseNMEA("$XCTRC*6d", nmea_info));
+
+  nmea_info.Reset();
+  nmea_info.clock = 1;
+
+  /* altitude and !wind */
+  ok1(device->ParseNMEA("$LXWP0,N,,1266.5,,,,,,,,248,23.1*55", nmea_info));
+  ok1(nmea_info.pressure_altitude_available);
+  ok1(!nmea_info.baro_altitude_available);
+  ok1(equals(nmea_info.pressure_altitude, 1266.5));
+
+  ok1(!nmea_info.airspeed_available);
+  ok1(!nmea_info.total_energy_vario_available);
+  ok1(!nmea_info.external_wind_available);
+
+  nmea_info.Reset();
+  nmea_info.clock = 1;
+
+  /* !airspeed and vario available */
+  ok1(device->ParseNMEA("$LXWP0,Y,222.3,1665.5,1.71,,,,,,239,174,10.1*47",nmea_info));
+  ok1(nmea_info.pressure_altitude_available);
+  ok1(!nmea_info.baro_altitude_available);
+  ok1(equals(nmea_info.pressure_altitude,1665.5));
+  ok1(!nmea_info.airspeed_available);
+
+  ok1(nmea_info.total_energy_vario_available);
+  ok1(equals(nmea_info.total_energy_vario, 1.71));
+
+  ok1(!nmea_info.external_wind_available);
+
+  /* the XCTRC sentence */
+  ok1(device->ParseNMEA("$XCTRC,2015,8,11,10,56,23,80,48.62825,8.104885,"
+      "129.4,11.01,322.76,-5.05,,,,997.79,77*53",nmea_info));
+
+  /* invalid date and time must be ignored */
+  ok1(device->ParseNMEA("$XCTRC,3015,13,33*77",nmea_info));
+  ok1(device->ParseNMEA("$XCTRC,,,,25,-1,99,33*69",nmea_info));
+
+  /* now check the correct values */
+  ok1(nmea_info.date_time_utc.year == 2015);
+  ok1(nmea_info.date_time_utc.month == 8);
+  ok1(nmea_info.date_time_utc.day == 11);
+  ok1(nmea_info.date_time_utc.hour == 10);
+  ok1(nmea_info.date_time_utc.minute == 56);
+  ok1(nmea_info.date_time_utc.second == 23);
+  ok1(equals(nmea_info.time, 10 * 3600 + 56 * 60 + 23.80));
+
+  ok1(nmea_info.location_available);
+  ok1(equals(nmea_info.location.longitude, 8.104885));
+  ok1(equals(nmea_info.location.latitude, 48.62825));
+
+  ok1(nmea_info.gps_altitude_available);
+  ok1(equals(nmea_info.gps_altitude, 129.4));
+
+  ok1(nmea_info.ground_speed_available);
+  ok1(equals(nmea_info.ground_speed, 11.01));
+
+  ok1(nmea_info.track_available);
+  ok1(equals(nmea_info.track,Angle::Degrees(322.76)));
+
+  ok1(nmea_info.total_energy_vario_available);
+  ok1(equals(nmea_info.total_energy_vario, -5.05));
+
+  ok1(nmea_info.battery_level_available);
+  ok1(equals(nmea_info.battery_level,77));
+
+  delete device;
+}
+
+static void
+TestACD()
+{
+  NullPort null;
+  Device *device = acd_driver.CreateOnPort(dummy_config, null);
+  ok1(device != NULL);
+
+  NMEAInfo nmea_info;
+
+  nmea_info.Reset();
+  nmea_info.clock = 1;
+
+  /* $PAAVS responses from XPDR must be ignored */
+  ok1(!device->ParseNMEA("$PAAVS,XPDR,7000,1,0,1697,0,0*68",nmea_info));
+
+  nmea_info.Reset();
+  nmea_info.clock = 1;
+
+  /* test ALT response */
+  ok1(device->ParseNMEA("$PAAVS,ALT,526.23,457.33,100500*0E",
+                        nmea_info));
+
+  ok1(nmea_info.pressure_altitude_available);
+  ok1(equals(nmea_info.pressure_altitude, 526.23));
+  ok1(nmea_info.baro_altitude_available);
+  ok1(equals(nmea_info.baro_altitude, 457.33));
+  ok1(nmea_info.settings.qnh_available);
+  ok1(equals(nmea_info.settings.qnh.GetPascal(), 100500));
+
+  nmea_info.Reset();
+  nmea_info.clock = 1;
+
+  /* test COM response */
+  ok1(device->ParseNMEA("$PAAVS,COM,130330,122500,100,75,1,1,0,0*0D",
+                        nmea_info));
+
+  ok1(nmea_info.settings.active_frequency.GetKiloHertz() == 130330);
+  ok1(nmea_info.settings.standby_frequency.GetKiloHertz() == 122500);
+  ok1(equals(nmea_info.settings.volume, 100));
+
+  delete device;
+}
 
 static void
 TestDeclare(const struct DeviceRegister &driver)
 {
-  FaultInjectionPort port(nullptr, *(DataHandler *)nullptr);
+  NullDataHandler handler;
+  FaultInjectionPort port(nullptr, handler);
   Device *device = driver.CreateOnPort(dummy_config, port);
   ok1(device != NULL);
 
@@ -1402,7 +1551,7 @@ TestDeclare(const struct DeviceRegister &driver)
                     Angle::Degrees(51.051944444444445));
   Waypoint wp(gp);
   wp.name = _T("Foo");
-  wp.elevation = fixed(123);
+  wp.elevation = 123;
   declaration.Append(wp);
   declaration.Append(wp);
   declaration.Append(wp);
@@ -1428,7 +1577,8 @@ TestDeclare(const struct DeviceRegister &driver)
 static void
 TestFlightList(const struct DeviceRegister &driver)
 {
-  FaultInjectionPort port(nullptr, *(DataHandler *)nullptr);
+  NullDataHandler handler;
+  FaultInjectionPort port(nullptr, handler);
   Device *device = driver.CreateOnPort(dummy_config, port);
   ok1(device != NULL);
 
@@ -1452,7 +1602,7 @@ TestFlightList(const struct DeviceRegister &driver)
 
 int main(int argc, char **argv)
 {
-  plan_tests(761);
+  plan_tests(827);
 
   TestGeneric();
   TestTasman();
@@ -1477,6 +1627,8 @@ int main(int argc, char **argv)
   TestZander();
   TestFlyNet();
   TestVaulter();
+  TestXCTracer();
+  TestACD();
 
   /* XXX the Triadis drivers have too many dependencies, not enabling
      for now */
